# -*- coding: utf-8 -*-
"""
utils
"""

from datetime import datetime
from time import mktime

import feedparser

from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import PermissionDenied

from ...models import BaseArticle
from ...settings import get_article_class

from .models import RssSource, RssItem


def collect_rss_items(user, source, check_user_rights=True):
    """
    download a rss feed and create rss_items
    the source can be a RssSource or any object with a get_absolute_url method
    """
    if check_user_rights and (not (user.is_staff and user.has_perm('rss_sync.add_rssitem'))):
        raise PermissionDenied

    feed = feedparser.parse(source.get_absolute_url())
    for entry in feed.entries:
        # create RSS entries if not exists
        item = RssItem.objects.get_or_create(link=entry.link, source=source)[0]
        # In any case, update the data
        item.title = entry.title
        try:
            item.updated = datetime.fromtimestamp(mktime(entry.updated_parsed))
        except AttributeError:
            item.updated = datetime.now()

        item.author = getattr(entry, 'author', '')[:100]
        item.summary = entry.summary
        item.save()

    if isinstance(source, RssSource):
        # update info for rss sources only
        source.title = getattr(feed.feed, 'title', '')
        source.last_collect = datetime.now()
        source.save()


def collect_all_rss_items():
    """collect all items"""
    all_models = getattr(settings, 'RSS_SYNC_SOURCE_MODELS', [RssSource])
    for model in all_models:
        for source in model.objects.all():
            collect_rss_items(None, source, check_user_rights=False)


def create_cms_article(user, item):
    """create a cms coop_cms.article from a RssItem"""
    article_class = get_article_class()
    content_type = ContentType.objects.get_for_model(article_class)
    perm = '{0}.add_{1}'.format(content_type.app_label, content_type.model)

    if not (user.is_staff and user.has_perm(perm)):
        raise PermissionDenied

    art = article_class.objects.create(title=item.title, content=item.summary, publication=BaseArticle.DRAFT)
    item.processed = True
    item.save()
    return art



# Generated by Django 4.2.16 on 2024-10-17 08:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('newsletters', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='contact',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='emailing',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='magiclink',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='subscriptiontype',
            name='allowed_on_sites',
            field=models.ManyToManyField(blank=True, related_name='+', to='sites.site'),
        ),
        migrations.AlterField(
            model_name='subscriptiontype',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]



# -*- coding:utf-8 -*-
"""urls"""

from django.conf import settings
from django.urls import include, path, re_path

from . import sitemap
from .settings import (
    get_article_views, install_csrf_failure_view, change_site_domain
)
from .views.newsletters import NewsletterView, NewsletterPdfView
from .views import articles, fragments, homepage, links, navigation, newsletters, medialib, webutils
from .views.webutils import DebugErrorCodeView

article_views = get_article_views()
article_view = article_views['article_view']
edit_article_view = article_views['edit_article_view']

install_csrf_failure_view()
change_site_domain()


urlpatterns = [
    path('htm-editor/', include('coop_html_editor.urls')),

    path('cms/change-template/<int:article_id>/', articles.change_template, name="coop_cms_change_template"),
    path('cms/settings/<int:article_id>/', articles.article_settings, name="coop_cms_article_settings"),
    path('cms/new/', articles.new_article, name="coop_cms_new_article"),
    path('cms/new/article/', articles.new_article, name="coop_cms_new_article"),
    path('cms/update-logo/<int:article_id>/', articles.update_logo, name="coop_cms_update_logo"),
    path('cms/articles/', articles.view_all_articles, name="coop_cms_view_all_articles"),
    path('cms/', articles.view_all_articles),
    path('articles/<slug:slug>/', articles.ArticlesByCategoryView.as_view(), name="coop_cms_articles_category"),

    path('cms/fragments/add/', fragments.add_fragment, name='coop_cms_add_fragment'),
    path('cms/fragments/edit/', fragments.edit_fragments, name='coop_cms_edit_fragments'),

    path('cms/set-homepage/<int:article_id>)/', homepage.set_homepage, name='coop_cms_set_homepage'),

    path('cms/new/link/', links.new_link, name="coop_cms_new_link"),

    path('cms/tree/<int:tree_id>/', navigation.process_nav_edition, name='navigation_tree'),

    path('cms/newsletter/new/', newsletters.newsletter_settings, name='coop_cms_new_newsletter'),
    path(
        'cms/newsletter/settings/<int:newsletter_id>/',
        newsletters.newsletter_settings,
        name='coop_cms_newsletter_settings'
    ),
    path(
        'cms/newsletter/<int:id>/',
        NewsletterView.as_view(),
        name='coop_cms_view_newsletter'
    ),
    path(
        'cms/newsletter-pdf/<int:id>/',
        NewsletterPdfView.as_view(),
        name='coop_cms_newsletter_pdf'
    ),
    path(
        'cms/newsletter/<int:id>/cms_edit/',
        NewsletterView.as_view(edit_mode=True),
        name='coop_cms_edit_newsletter'
    ),
    path(
        'cms/newsletter/change-template/<int:newsletter_id>/',
        newsletters.change_newsletter_template,
        name="coop_cms_change_newsletter_template"
    ),
    path(
        'cms/newsletter/test/<int:newsletter_id>/',
        newsletters.test_newsletter,
        name="coop_cms_test_newsletter"
    ),
    path(
        'cms/newsletter/schedule/<int:newsletter_id>/',
        newsletters.schedule_newsletter_sending,
        name="coop_cms_schedule_newsletter_sending"
    ),

    path('cms/media-images/', medialib.show_media, {'media_type': 'image'}, name='coop_cms_media_images'),
    path('cms/media-documents/', medialib.show_media, {'media_type': 'document'}, name='coop_cms_media_documents'),
    path(
        'cms/media-photologue/',
        medialib.show_media,
        {'media_type': 'photologue'},
        name='coop_cms_media_photologue'
    ),
    path('cms/upload-image/', medialib.upload_image, name="coop_cms_upload_image"),
    path('cms/upload-doc/', medialib.upload_doc, name="coop_cms_upload_doc"),
    path('cms/private-download/<int:doc_id>/', medialib.download_doc, name='coop_cms_download_doc'),

    path('cms/change-language/', webutils.change_language, name='coop_cms_change_language'),
    path('cms/swicth-language/', webutils.switch_language_popup, name='coop_cms_switch_language_popup'),

    path(
        'cms/accept-cookies-message/',
        webutils.accept_cookies_message,
        name='coop_cms_accept_cookies_message'
    ),
]

if settings.DEBUG:
    urlpatterns += [
        re_path(
            r'^cms/debug-error-code/(?P<error_code>\d{3})/$',
            DebugErrorCodeView.as_view(),
            name='coop_cms_debug_404'
        ),
    ]

if not getattr(settings, "COOP_CMS_DISABLE_DEFAULT_SITEMAP", False):
    urlpatterns += sitemap.urlpatterns

if 'coop_cms.apps.rss_sync' in settings.INSTALLED_APPS:
    urlpatterns += [
        path('rss-sync/', include('coop_cms.apps.rss_sync.urls')),
    ]

if 'coop_cms.apps.test_app' in settings.INSTALLED_APPS:
    urlpatterns += [
        path('this-should-be-only-in-test-mode', include('coop_cms.apps.test_app.urls')),
    ]

# keep these at the end
urlpatterns += [
    re_path(r'(?P<url>[-\w]+)/cms_publish/$', articles.publish_article, name='coop_cms_publish_article'),
    re_path(r'^(?P<url>[-\w]+)/cms_cancel/$', articles.cancel_edit_article, name='coop_cms_cancel_edit_article'),
    path('', homepage.homepage, name='coop_cms_homepage'),
    re_path(r'^(?P<slug>[-\w]+)/cms_edit/$', edit_article_view.as_view(edit_mode=True), name='coop_cms_edit_article'),
    re_path(r'^(?P<slug>[-\w]+)/$', article_view.as_view(), name='coop_cms_view_article'),
    re_path(r'^(?P<path>.+)$', articles.AliasView.as_view(), name='coop_cms_view_alias'),
    path('coop_bar/', include('coop_bar.urls')),
]



# -*- coding: utf-8 -*-

from django.urls import path

from . import views

app_name = "newsletters"


urlpatterns = [
    path(
        'unregister/<int:emailing_id>/<uuid:contact_uuid>/',
        views.unregister_contact,
        name='unregister'
    ),
    path(
        'view-online/<int:emailing_id>/<uuid:contact_uuid>/',
        views.view_emailing_online,
        name='view_online'
    ),
    path(
        'view-online-lang/<int:emailing_id>/<uuid:contact_uuid>/<str:lang>/',
        views.view_emailing_online_lang,
        name='view_online_lang'
    ),
    path(
        'link/<uuid:link_uuid>/<uuid:contact_uuid>/',
        views.view_link,
        name='view_link'
    ),
    path(
        'email-img/<int:emailing_id>/<uuid:contact_uuid>/',
        views.email_tracking,
        name='email_tracking'
    ),
]



# -*- coding: utf-8 -*-

from django.conf import settings
from django.urls import reverse

from model_mommy import mommy

from ..settings import get_article_class
from . import BaseArticleTest


class TemplateTest(BaseArticleTest):
    """It should display articles with the right template"""

    def setUp(self):
        super(TemplateTest, self).setUp()
        self._default_article_templates = settings.COOP_CMS_ARTICLE_TEMPLATES
        settings.COOP_CMS_ARTICLE_TEMPLATES = (
            ('coop_cms/article.html', 'coop_cms base article'),
            ('test/article.html', 'test article'),
        )
        
    def tearDown(self):
        super(TemplateTest, self).tearDown()
        # restore
        settings.COOP_CMS_ARTICLE_TEMPLATES = self._default_article_templates

    def test_view_article(self):
        """Check that we are do not using the PrivateArticle anymore"""
        article_class = get_article_class()
        article = mommy.make(article_class, slug="test", publication=article_class.PUBLISHED)
        response = self.client.get(article.get_absolute_url())
        self.assertTemplateUsed(response, 'coop_cms/article.html')
        self.assertEqual(200, response.status_code)
        
    def test_view_article_custom_template(self):
        """Check that we are do not using the PrivateArticle anymore"""
        article_class = get_article_class()
        article = mommy.make(article_class, slug="test", publication=article_class.PUBLISHED, template='test/article.html')
        response = self.client.get(article.get_absolute_url())
        self.assertTemplateUsed(response, 'test/article.html')
        self.assertEqual(200, response.status_code)
        
    def test_change_template(self):
        """Check that we are do not using the PrivateArticle anymore"""
        article_class = get_article_class()
        article = mommy.make(article_class, slug="test")
        self._log_as_editor()
        url = reverse('coop_cms_change_template', args=[article.id])
        response = self.client.post(url, data={'template': 'test/article.html'}, follow=True)
        self.assertEqual(200, response.status_code)
        article = article_class.objects.get(id=article.id)#refresh
        self.assertEqual(article.template, 'test/article.html')
        
    def test_change_template_permission(self):
        """Check that we are do not using the PrivateArticle anymore"""
        article_class = get_article_class()
        article = mommy.make(article_class, slug="test")
        url = reverse('coop_cms_change_template', args=[article.id])
        response = self.client.post(url, data={'template': 'test/article.html'})
        self.assertNotAllowed(response)
        article = article_class.objects.get(id=article.id)  # refresh
        self.assertEqual(article.template, '')



# -*- coding: utf-8 -*-

from ..utils import RequestManager, RequestMiddleware, RequestNotFound
from . import BaseTestCase


class RequestManagerTest(BaseTestCase):
    """Test request manager"""
    
    def test_get_request(self):
        """retrieve request"""
        request1 = {'user': "joe"}
        def fake_view(*args, **kwargs):
            request2 = RequestManager().get_request()
            self.assertEqual(request1, request2)
        RequestMiddleware(fake_view)(request1)

    def test_get_request_no_middleware(self):
        """if no request"""
        RequestManager().clean()
        self.assertRaises(RequestNotFound, RequestManager().get_request)



# Generated by Django 4.2.20 on 2025-10-02 08:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('newsletters', '0003_alter_contact_id_alter_emailing_id_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='subscription',
            name='unsubscription_reason',
            field=models.TextField(blank=True, default='', verbose_name='unsubscription reason'),
        ),
    ]



# -*- coding: utf-8 -*-
"""forms"""

import floppyforms as floppyforms

from coop_html_editor.settings import init_url, ckeditor_version
from coop_html_editor.widgets import CkEditorInput, Media, logging


class CoopCkEditorInput(CkEditorInput):

    def __init__(self, kind, *args, **kwargs):
        self.kind = kind
        super().__init__(*args, **kwargs)

    @property
    def media(self):
        try:
            css = {'all': ()}
            js_files = [
                '{0}/ckeditor.js'.format(ckeditor_version()),
                init_url() + ('?kind=' + self.kind if self.kind else ''),
            ]
            return Media(css=css, js=js_files)
        except Exception as msg:
            django_logger = logging.getLogger('django')
            django_logger.error(u'CkEditorInput._get_media Error {0}'.format(msg))


class InlineHtmlEditableModelForm(floppyforms.ModelForm):
    """Base class for form with inline-HTML editor fields"""
    is_inline_editable = True  # The cms_edition templatetag checks this for switching to edit mode
    kind = ''

    def __init__(self, *args, **kwargs):
        super(InlineHtmlEditableModelForm, self).__init__(*args, **kwargs)  # pylint: disable=E1002
        for field_name in self.Meta.fields:
            no_inline_html_widgets = getattr(self.Meta, 'no_inline_editable_widgets', ())
            if field_name not in no_inline_html_widgets:
                self.fields[field_name].widget = CoopCkEditorInput(self.kind)

    class Media:
        css = {
            'all': ('css/colorbox.css', ),
        }
        js = (
            'js/jquery.form.js',
            'js/jquery.pageslide.js',
            'js/jquery.colorbox-min.js',
            'js/colorbox.coop.js',
        )



# -*- coding: utf-8 -*-
"""test email sending"""

from datetime import datetime

from django.conf import settings
from django.contrib.sites.models import Site
from django.core import management
from django.core import mail

from model_mommy import mommy

from ....models import Newsletter
from ....tests import BaseTestCase

from .. import models


class CustomTemplateTest(BaseTestCase):
    """Test the user-templating"""

    def test_send_newsletter(self):
        contact = mommy.make(models.Contact, email='toto@toto.fr', first_name='Emmet', last_name='Brown')
        settings.COOP_CMS_FROM_EMAIL = 'toto@toto.fr'

        content = '''
        <h2>Hello #!-first_name-!# #!-last_name-!#</h2>
        Email: #!-email-!#
        '''

        newsletter_data = {
            'subject': 'Hello #!-first_name-!#',
            'content': content,
            'template': 'test/newsletter_contact.html',
        }

        newsletter = mommy.make(Newsletter, **newsletter_data)

        site = Site.objects.get_current()
        site.domain = "toto.fr"
        site.save()

        emailing = mommy.make(
            models.Emailing,
            newsletter=newsletter,
            status=models.Emailing.STATUS_SCHEDULED,
            scheduling_dt=datetime.now(),
            sending_dt=None,
            subscription_type=mommy.make(models.SubscriptionType, site=site),
        )

        emailing.send_to.add(contact)
        emailing.save()

        management.call_command('send_newsletters', verbosity=0)

        emailing = models.Emailing.objects.get(id=emailing.id)

        # Check emailing status
        self.assertEqual(emailing.status, models.Emailing.STATUS_SENT)
        self.assertNotEqual(emailing.sending_dt, None)
        self.assertEqual(emailing.send_to.count(), 0)
        self.assertEqual(emailing.sent_to.count(), 1)

        self.assertEqual(len(mail.outbox), 1)

        email = mail.outbox[0]

        self.assertEqual(email.to, [contact.email])
        self.assertEqual(email.from_email, settings.COOP_CMS_FROM_EMAIL)

        self.assertTrue(email.alternatives[0][1], "text/html")

        html_body = email.alternatives[0][0]
        self.assertTrue(html_body.find(contact.fullname) >= 0)

        self.assertEqual(email.subject, 'Hello Emmet')

        fields = ('first_name', 'last_name', 'email',)
        for field_name in fields:
            self.assertTrue(email.body.find(getattr(contact, field_name)) >= 0)
            self.assertTrue(html_body.find(getattr(contact, field_name)) >= 0)



# Generated by Django 4.2 on 2023-05-22 13:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('coop_cms', '0018_auto_20201209_0944'),
    ]

    operations = [
        migrations.AlterField(
            model_name='navtree',
            name='types',
            field=models.ManyToManyField(blank=True, related_name='%(app_label)s_%(class)s_set', to='coop_cms.navtype'),
        ),
    ]



# -*- coding: utf-8 -*-
"""
Basic app for article
"""

from django.apps import AppConfig


class BasicCmsAppConfig(AppConfig):
    name = 'coop_cms.apps.basic_cms'
    verbose_name = "coop CMS > Basic CMS"



# -*- coding: utf-8 -*-

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('basic_cms', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='article',
            name='login_required',
            field=models.BooleanField(default=False, help_text='If true, only user with login/password will able to access the article', verbose_name='login required'),
        ),
    ]



# -*- coding: utf-8 -*-
"""
Admin site
"""

from django.contrib import admin

from ...admin import ArticleAdmin as CmsArticleAdmin
from ...settings import get_article_class


article_class = get_article_class()

# Replace the default Article admin
admin.site.unregister(article_class)


@admin.register(article_class)
class ArticleAdmin(CmsArticleAdmin):
    """Custom Article admin"""
    fieldsets = CmsArticleAdmin.fieldsets + (
        ('Misc', {'fields': ('author',)}),
    )



# -*- coding: utf-8 -*-
"""articles"""

import json

from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.contrib.contenttypes.models import ContentType
from django.contrib.messages.api import success as success_message
from django.core.exceptions import PermissionDenied
from django.http import HttpResponse, Http404, HttpResponseRedirect
from django.shortcuts import render, get_object_or_404
from django.template.exceptions import TemplateDoesNotExist
from django.template.loader import get_template
from django.urls import reverse
from django.views.generic.base import TemplateView
from django.utils.translation import gettext as _
from django.views.generic import View

from colorbox.decorators import popup_redirect

from ..exceptions import ArticleNotAllowed
from ..forms.articles import ArticleLogoForm, ArticleTemplateForm, PublishArticleForm
from .. import models
from ..generic_views import EditableObjectView
from ..logger import logger
from ..moves import is_authenticated
from ..settings import (
    get_article_class, get_article_form, get_article_settings_form, get_new_article_form,
    get_articles_category_page_size, homepage_no_redirection
)
from ..shortcuts import get_article_or_404, get_headlines, redirect_if_alias
from ..utils import get_model_name, get_model_app, paginate


def get_article_template(article):
    """get article template"""
    template = article.template
    if not template:
        template = 'coop_cms/article.html'
    return template


@login_required
def view_all_articles(request):
    """all article"""

    add_article_url = articles_admin_url = None
    article_class = get_article_class()
    content_type = ContentType.objects.get_for_model(article_class)
    articles = []
    if request.user.is_authenticated:
        view_perm = '{0}.view_{1}'.format(content_type.app_label, content_type.model)
        if request.user.has_perm(view_perm):
            articles = article_class.objects.filter(sites__id=settings.SITE_ID).order_by('-id')[:20]
        if request.user.is_staff:
            view_name = 'admin:{0}_{1}_changelist'.format(get_model_app(article_class), get_model_name(article_class))
            articles_admin_url = reverse(view_name)

    add_perm = '{0}.add_{1}'.format(content_type.app_label, content_type.model)
    if request.user.has_perm(add_perm):
        add_article_url = reverse('coop_cms_new_article')

    return render(
        request,
        'coop_cms/view_all_articles.html',
        {
            'articles': articles,
            'editable': True,
            'articles_list_url': articles_admin_url,
            'add_article_url': add_article_url,
        }
    )


@login_required
def cancel_edit_article(request, url):
    """if cancel_edit, delete the preview image"""
    article = get_article_or_404(slug=url, sites=settings.SITE_ID)
    if article.temp_logo:
        article.temp_logo = ''
        article.save()
    return HttpResponseRedirect(article.get_absolute_url())


@login_required
@popup_redirect
def publish_article(request, url):
    """change the publication status of an article"""
    article = get_article_or_404(slug=url, sites=settings.SITE_ID)

    if not request.user.has_perm('can_publish_article', article):
        raise PermissionDenied

    draft = (article.publication == models.BaseArticle.DRAFT)
    if draft:
        article.publication = models.BaseArticle.PUBLISHED
    else:
        article.publication = models.BaseArticle.DRAFT

    if request.method == "POST":
        form = PublishArticleForm(request.POST, instance=article)
        if form.is_valid():
            article = form.save()
            return HttpResponseRedirect(article.get_absolute_url())
    else:
        form = PublishArticleForm(instance=article)

    context_dict = {
        'form': form,
        'article': article,
        'draft': draft,
        'title': _("Do you want to publish this article?") if draft else _("Make it draft?"),
    }

    return render(
        request,
        'coop_cms/popup_publish_article.html',
        context_dict
    )


@login_required
@popup_redirect
def change_template(request, article_id):
    """change template"""

    article = get_object_or_404(get_article_class(), id=article_id)
    if request.method == "POST":
        form = ArticleTemplateForm(article, request.user, request.POST, request.FILES)
        if form.is_valid():
            article.template = form.cleaned_data['template']
            article.save()
            return HttpResponseRedirect(article.get_edit_url())
    else:
        form = ArticleTemplateForm(article, request.user)

    return render(
        request,
        'coop_cms/popup_change_template.html',
        locals()
    )


@login_required
@popup_redirect
def article_settings(request, article_id):
    """article settings"""
    article = get_object_or_404(get_article_class(), id=article_id)
    article_settings_form_class = get_article_settings_form()

    if not request.user.has_perm('can_edit_article', article):
        raise PermissionDenied

    if request.method == "POST":
        form = article_settings_form_class(request.user, request.POST, request.FILES, instance=article)
        if form.is_valid():
            article = form.save()
            form.save_m2m()
            return HttpResponseRedirect(article.get_absolute_url())
    else:
        form = article_settings_form_class(request.user, instance=article)

    context = {
        'article': article,
        'form': form,
    }
    return render(
        request,
        'coop_cms/popup_article_settings.html',
        context
    )


@login_required
@popup_redirect
def new_article(request):
    """new article"""
    article_class = get_article_class()
    new_article_form = get_new_article_form()

    content_type = ContentType.objects.get_for_model(article_class)
    perm = '{0}.add_{1}'.format(content_type.app_label, content_type.model)

    if not request.user.has_perm(perm):
        raise PermissionDenied

    if request.method == "POST":
        form = new_article_form(request.user, request.POST, request.FILES)
        if form.is_valid():
            article = form.save()
            form.save_m2m()
            success_message(request, _('The article has been created properly'))
            return HttpResponseRedirect(article.get_edit_url())
    else:
        form = new_article_form(request.user)

    return render(
        request,
        'coop_cms/popup_new_article.html',
        locals()
    )


@login_required
def update_logo(request, article_id):
    """update logo"""
    try:
        article = get_object_or_404(get_article_class(), id=article_id)
        if request.method == "POST":
            form = ArticleLogoForm(request.POST, request.FILES)
            if form.is_valid():
                article.temp_logo = form.cleaned_data['image']
                article.save()
                url = article.logo_thumbnail(True).url
                data = {'ok': True, 'src': url}
                return HttpResponse(json.dumps(data), content_type='application/json')
            else:
                template = get_template('coop_cms/popup_update_logo.html')
                html = template.render(locals())
                data = {'ok': False, 'html': html}
                return HttpResponse(json.dumps(data), content_type='application/json')
        else:
            form = ArticleLogoForm()

        return render(
            request,
            'coop_cms/popup_update_logo.html',
            locals()
        )
    except Exception:
        logger.exception("update_logo")
        raise


class ArticlesByCategoryView(TemplateView):
    """Show the articles of a given category"""
    category = None

    def get_category(self):
        """return the category"""
        if not self.category:
            slug = self.kwargs['slug']
            self.category = get_object_or_404(models.ArticleCategory, slug=slug, sites__id=settings.SITE_ID)
        return self.category

    def get_articles(self, category):
        """return list of articles for this category, 404 if no articles"""
        articles = category.get_articles_qs().filter(
            publication=models.BaseArticle.PUBLISHED
        ).order_by("-publication_date")

        if articles.count() == 0:
            raise Http404

        return articles

    def get_context_data(self, **kwargs):
        """context"""
        context_data = super(ArticlesByCategoryView, self).get_context_data()
        category = self.get_category()

        if not self.request.user.has_perm('can_view_category', category):
            raise PermissionDenied()

        articles = self.get_articles(category)

        page_obj = paginate(self.request, articles, get_articles_category_page_size(category))

        context_data.update({
            'category': category,
            "articles": list(page_obj),
            'page_obj': page_obj,
        })
        return context_data

    def get_template_names(self):
        """template to use"""
        try:
            category_template = "coop_cms/categories/{0}.html".format(self.get_category().slug)
            get_template(category_template)
        except TemplateDoesNotExist:
            category_template = "coop_cms/articles_category.html"

        return [category_template]


class ArticleView(EditableObjectView):
    """Article view for edition"""
    model = get_article_class()
    form_class = get_article_form()
    field_lookup = "slug"
    varname = "article"
    as_homepage = False

    def get_object(self):
        """get object"""
        article = get_article_or_404(slug=self.kwargs['slug'], sites=settings.SITE_ID)
        if not self.edit_mode:
            if article.is_homepage and homepage_no_redirection() and not self.as_homepage:
                # Do not authorize access to homepage with its urls in 'homepage_no_redirection' mode
                raise ArticleNotAllowed
        return article

    def dispatch(self, request, *args, **kwargs):
        try:
            return super(ArticleView, self).dispatch(request, *args, **kwargs)
        except Http404:
            slug = self.kwargs['slug']
            return redirect_if_alias(slug)
        except ArticleNotAllowed:
            raise Http404

    def can_access_object(self):
        """perms -> 404 if no perms"""

        if self.object.login_required and not is_authenticated(self.request.user):
            raise PermissionDenied

        if self.object.is_archived():
            return super(ArticleView, self).can_view_object()
        return True

    def get_headlines(self):
        """headline"""
        return get_headlines(self.object)

    def get_context_data(self):
        """context"""
        context_data = super(ArticleView, self).get_context_data()
        context_data.update({
            'draft': self.object.publication == models.BaseArticle.DRAFT,
            'headlines': self.get_headlines(),
            'ARTICLE_PUBLISHED': models.BaseArticle.PUBLISHED
        })
        return context_data

    def after_save(self, article):
        """after save"""
        if article.temp_logo:
            article.logo = article.temp_logo
            article.temp_logo = ''
            article.save()

    def get_template(self):
        """get template"""
        return get_article_template(self.object)


class AliasView(View):
    """redirect to another url"""

    def get(self, *args, **kwargs):
        """redirect to aliased page"""
        return redirect_if_alias(path=kwargs.get('path'))



# -*- coding: utf-8 -*-
"""Every code which depends from optional apps"""

from django.conf import settings


def not_implemented(*args, **kwargs):
    raise NotImplemented


if 'modeltranslation' in settings.INSTALLED_APPS:
    from modeltranslation.utils import build_localized_fieldname
else:
    build_localized_fieldname = lambda x, y: x


if 'wkhtmltopdf' in settings.INSTALLED_APPS:
    from wkhtmltopdf.utils import convert_to_pdf, make_absolute_paths
    from wkhtmltopdf.views import PDFResponse
else:
    convert_to_pdf = not_implemented
    make_absolute_paths = not_implemented
    PDFResponse = not_implemented



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-07-19 14:08

import coop_cms.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('basic_cms', '0004_auto_20161031_1401'),
    ]

    operations = [
        migrations.AlterField(
            model_name='article',
            name='content',
            field=models.TextField(blank=True, default='', verbose_name='content'),
        ),
        migrations.AlterField(
            model_name='article',
            name='logo',
            field=models.ImageField(blank=True, default='', null=True, upload_to=coop_cms.models.get_logo_folder),
        ),
        migrations.AlterField(
            model_name='article',
            name='sites',
            field=models.ManyToManyField(default=[], to='sites.Site', verbose_name='site'),
        ),
        migrations.AlterField(
            model_name='article',
            name='subtitle',
            field=models.TextField(blank=True, default='', verbose_name='subtitle'),
        ),
        migrations.AlterField(
            model_name='article',
            name='summary',
            field=models.TextField(blank=True, default='', verbose_name='Summary'),
        ),
        migrations.AlterField(
            model_name='article',
            name='temp_logo',
            field=models.ImageField(blank=True, default='', null=True, upload_to=coop_cms.models.get_logo_folder),
        ),
        migrations.AlterField(
            model_name='article',
            name='template',
            field=models.CharField(blank=True, default='', max_length=200, verbose_name='template'),
        ),
        migrations.AlterField(
            model_name='article',
            name='title',
            field=models.TextField(blank=True, default='', verbose_name='title'),
        ),
        migrations.AlterField(
            model_name='navtree',
            name='name',
            field=models.CharField(db_index=True, default='default', max_length=100, unique=True, verbose_name='name'),
        ),
    ]



# -*- coding: utf-8 -*-
"""navigation: the navigation tree page"""

import json

from django.db.models.aggregates import Max
from django.contrib.auth.decorators import login_required
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError, PermissionDenied
from django.http import HttpResponse, Http404
from django.shortcuts import get_object_or_404
from django.template.loader import select_template
from django.urls import reverse
from django.utils.translation import gettext as _

from .. import models
from ..moves import make_context
from ..settings import get_navtree_class
from ..logger import logger
from ..utils import get_model_app, get_model_label, get_model_name


def view_navnode(request, tree):
    """show info about the node when selected"""
    try:
        response = {}

        node_id = request.POST['node_id']
        node = models.NavNode.objects.get(tree=tree, id=node_id)
        model_name = object_label = ""

        # get the admin url
        if node.content_type:
            app, mod = node.content_type.app_label, node.content_type.model
            admin_url = reverse("admin:{0}_{1}_change".format(app, mod), args=(node.object_id,))

            # load and render template for the object
            # try to load the corresponding template and if not found use the default one
            model_name = "{0}".format(node.content_type)
            object_label = "{0}".format(node.content_object)
            template = select_template([
                "coop_cms/navtree_content/{0}.html".format(node.content_type.name),
                "coop_cms/navtree_content/default.html"
            ])
        else:
            admin_url = ""
            template = select_template(["coop_cms/navtree_content/default.html"])

        context_dict = {
            "node": node,
            "admin_url": admin_url,
            "model_name": model_name,
            "object_label": object_label,
        }

        html = template.render(
            make_context(request, context_dict)
        )

        # return data has dictionary
        response['html'] = html
        response['message'] = "Node content loaded."

        return response
    except Exception:
        logger.exception("view_navnode")
        raise


def rename_navnode(request, tree):
    """change the name of a node when renamed in the tree"""
    response = {}
    node_id = request.POST['node_id']
    node = models.NavNode.objects.get(tree=tree, id=node_id)  # get the node
    old_name = node.label  # get the old name for success message
    node.label = request.POST['name']  # change the name
    node.save()
    if old_name != node.label:
        response['message'] = _("The node '{0}' has been renamed into '{1}'.").format(old_name, node.label)
    else:
        response['message'] = ''
    return response


def remove_navnode(request, tree):
    """delete a node"""
    # Keep multi node processing even if multi select is not allowed
    response = {}
    node_ids = request.POST['node_ids'].split(";")
    for node_id in node_ids:
        models.NavNode.objects.get(tree=tree, id=node_id).delete()
    if len(node_ids) == 1:
        response['message'] = _("The node has been removed.")
    else:
        response['message'] = _("{0} nodes has been removed.").format(len(node_ids))
    return response


def move_navnode(request, tree):
    """move a node in the tree"""
    response = {}

    node_id = request.POST['node_id']
    ref_pos = request.POST['ref_pos']
    parent_id = request.POST.get('parent_id', 0)
    ref_id = request.POST.get('ref_id', 0)

    node = models.NavNode.objects.get(tree=tree, id=node_id)

    if parent_id:
        sibling_nodes = models.NavNode.objects.filter(tree=tree, parent__id=parent_id)
        parent_node = models.NavNode.objects.get(tree=tree, id=parent_id)
    else:
        sibling_nodes = models.NavNode.objects.filter(tree=tree, parent__isnull=True)
        parent_node = None

    if ref_id:
        ref_node = models.NavNode.objects.get(tree=tree, id=ref_id)
    else:
        ref_node = None

    # Update parent if changed
    if parent_node != node.parent:
        if node.parent:
            ex_siblings = models.NavNode.objects.filter(tree=tree, parent=node.parent).exclude(id=node.id)
        else:
            ex_siblings = models.NavNode.objects.filter(tree=tree, parent__isnull=True).exclude(id=node.id)

        node.parent = parent_node

        # restore ex-siblings
        for sib_node in ex_siblings.filter(ordering__gt=node.ordering):
            sib_node.ordering -= 1
            sib_node.save()

        # move siblings if inserted
        if ref_node:
            if ref_pos == "before":
                to_be_moved = sibling_nodes.filter(ordering__gte=ref_node.ordering)
                node.ordering = ref_node.ordering
            elif ref_pos == "after":
                to_be_moved = sibling_nodes.filter(ordering__gt=ref_node.ordering)
                node.ordering = ref_node.ordering + 1
            for ntbm in to_be_moved:
                ntbm.ordering += 1
                ntbm.save()

        else:
            # add at the end
            max_ordering = sibling_nodes.aggregate(max_ordering=Max('ordering'))['max_ordering'] or 0
            node.ordering = max_ordering + 1

    else:

        # Update pos if changed
        if ref_node:
            if ref_node.ordering > node.ordering:
                # move forward
                to_be_moved = sibling_nodes.filter(ordering__lt=ref_node.ordering, ordering__gt=node.ordering)
                for next_sibling_node in to_be_moved:
                    next_sibling_node.ordering -= 1
                    next_sibling_node.save()

                if ref_pos == "before":
                    node.ordering = ref_node.ordering - 1

                elif ref_pos == "after":
                    node.ordering = ref_node.ordering - 1

            elif ref_node.ordering < node.ordering:
                # move backward
                to_be_moved = sibling_nodes.filter(ordering__gt=ref_node.ordering, ordering__lt=node.ordering)
                for next_sibling_node in to_be_moved:
                    next_sibling_node.ordering += 1
                    next_sibling_node.save()

                if ref_pos == "before":
                    node.ordering = ref_node.ordering
                    ref_node.ordering += 1
                    ref_node.save()
                elif ref_pos == "after":
                    node.ordering = ref_node.ordering + 1

        else:
            max_ordering = sibling_nodes.aggregate(max_ordering=Max('ordering'))['max_ordering'] or 0
            node.ordering = max_ordering + 1

    node.save()
    response['message'] = _("The node '{0}' has been moved.").format(node.label)

    return response


def add_navnode(request, tree):
    """Add a new node"""
    response = {}

    # get the type of object
    object_type = request.POST['object_type']
    if object_type:
        app_label, model_name = object_type.split('.')
        content_type = ContentType.objects.get(app_label=app_label, model=model_name)
        model_class = content_type.model_class()
        object_id = request.POST['object_id']
        model_name = get_model_label(model_class)
        if not object_id:
            raise ValidationError(_("Please choose an existing {0}").format(model_name.lower()))
        try:
            obj = model_class.objects.get(id=object_id)
        except model_class.DoesNotExist:
            raise ValidationError(_("{0} {1} not found").format(get_model_label(model_class), object_id))

        # objects can not be added twice in the navigation tree
        if models.NavNode.objects.filter(tree=tree, content_type=content_type, object_id=obj.id).count() > 0:
            raise ValidationError(_("The {0} is already in navigation").format(get_model_label(model_class)))

    else:
        content_type = None
        obj = None

    # Create the node
    parent_id = request.POST.get('parent_id', 0)
    if parent_id:
        parent = models.NavNode.objects.get(tree=tree, id=parent_id)
    else:
        parent = None
    node = models.create_navigation_node(content_type, obj, tree, parent)

    response['label'] = node.label
    response['id'] = 'node_{0}'.format(node.id)
    response['message'] = _("'{0}' has added to the navigation tree.").format(node.label)

    return response


def get_suggest_list(request, tree):
    """call by auto-complete"""
    response = {}
    suggestions = []
    term = request.POST["term"]  # the 1st chars entered in the autocomplete

    if tree.types.count() == 0:
        nav_types = models.NavType.objects.all()
    else:
        nav_types = tree.types.all()

    for nav_type in nav_types:
        content_type = nav_type.content_type
        if nav_type.label_rule == models.NavType.LABEL_USE_SEARCH_FIELD:
            # Get the name of the default field for the current type (eg: Page->title, Url->url ...)
            lookup = {nav_type.search_field + '__icontains': term}
            objects = content_type.model_class().objects.filter(**lookup)
        elif nav_type.label_rule == models.NavType.LABEL_USE_GET_LABEL:
            objects = [
                obj for obj in content_type.model_class().objects.all() if term.lower() in obj.get_label().lower()
            ]
        else:
            objects = [
                obj for obj in content_type.model_class().objects.all() if term.lower() in '{0}'.format(obj).lower()
            ]

        already_in_navigation = [
            node.object_id for node in models.NavNode.objects.filter(tree=tree, content_type=content_type)
        ]

        # Get suggestions as a list of {label: object.get_label() or unicode if no get_label, 'value':<object.id>}
        for obj in objects:
            if obj.id not in already_in_navigation:
                # Suggest only articles which are not in navigation yet
                suggestions.append({
                    'label': models.get_object_label(content_type, obj),
                    'value': obj.id,
                    'category': get_model_label(content_type.model_class()).capitalize(),
                    'type': content_type.app_label + '.' + content_type.model,
                })

    # Add suggestion for an empty node
    suggestions.append({
        'label': _("Node"),
        'value': 0,
        'category': _("Empty node"),
        'type': "",
    })
    response['suggestions'] = suggestions
    return response


def navnode_in_navigation(request, tree):
    """toogle the is_visible_flag of a navnode"""
    response = {}
    node_id = request.POST['node_id']
    node = models.NavNode.objects.get(tree=tree, id=node_id)  # get the node
    node.in_navigation = not node.in_navigation
    node.save()
    if node.in_navigation:
        response['message'] = _("The node is now visible.")
        response['label'] = _("Hide node in navigation")
        response['icon'] = "in_nav"
    else:
        response['message'] = _("The node is now hidden.")
        response['label'] = _("Show node in navigation")
        response['icon'] = "out_nav"
    return response


@login_required
def process_nav_edition(request, tree_id):
    """This handle ajax request sent by the tree component"""
    if request.method == 'POST' and 'msg_id' in request.POST:
        try:
            # Get the current tree
            tree_class = get_navtree_class()
            tree = get_object_or_404(tree_class, id=tree_id)

            # check permissions
            perm_name = "{0}.change_{1}".format(get_model_app(tree_class), get_model_name(tree_class))
            if not request.user.has_perm(perm_name):
                raise PermissionDenied

            functions = (
                view_navnode, rename_navnode, remove_navnode, move_navnode,
                add_navnode, get_suggest_list, navnode_in_navigation,
            )
            supported_msg = {}
            # create a map between message name and handler
            # use the function name as message id
            for fct in functions:
                supported_msg[fct.__name__] = fct

            # Call the handler corresponding to the requested message
            response = supported_msg[request.POST['msg_id']](request, tree)

            # If no exception raise: Success
            response['status'] = 'success'
            response.setdefault('message', 'Ok')  # if no message defined in response, add something

        except KeyError as msg:
            response = {'status': 'error', 'message': "Unsupported message : {0}".format(msg)}
        except PermissionDenied:
            response = {'status': 'error', 'message': "You are not allowed to add a node"}
        except ValidationError as ex:
            response = {'status': 'error', 'message': ' - '.join(ex.messages)}
        except Exception as msg:
            logger.exception("process_nav_edition")
            response = {'status': 'error', 'message': "An error occured : {0}".format(msg)}

        # return the result as json object
        return HttpResponse(json.dumps(response), content_type='application/json')
    raise Http404



# -*- coding: utf-8 -*-
"""Admin forms"""

from django import forms
from django.conf import settings

from . import models, widgets

# These 2 admin forms makes possible to add custom widgets for the id field


class RssSourceAdminForm(forms.ModelForm):
    """Admin form"""
    url = forms.CharField(widget=widgets.AdminCollectRssWidget)

    class Meta:
        model = models.RssSource
        fields = ('url', 'title', 'last_collect')

    class Media:
        css = {
            'all': (settings.STATIC_URL + 'css/rss_sync/admin-cust.css',),
        }


class RssItemAdminForm(forms.ModelForm):
    """Admin form"""
    # id = forms.IntegerField(widget=widgets.AdminCreateArticleWidget, label='Id')

    class Meta:
        model = models.RssItem
        fields = ('id', 'source', 'link', 'title', 'summary', 'author', 'updated', 'processed')
        widgets = {
            'id': widgets.AdminCreateArticleWidget,
        }



# -*- coding: utf-8 -*-
"""common place for settings. Define defaults"""

from django.utils.translation import gettext as _
from django.conf import settings as project_settings


def get_language_choices(default_label=None):
    """returns list of languages"""
    if default_label is None:
        default_label = _('Default')
    return [('', default_label)] + list(project_settings.LANGUAGES)


def has_language_choices():
    """returns true if we should propose language choices"""
    return len(project_settings.LANGUAGES) >= 2


def get_ignored_magic_links():
    """returns true if we should propose language choices"""
    return getattr(project_settings, 'COOP_CMS_NEWSLETTER_IGNORED_MAGIC_LINKS', [])



# -*- coding: utf-8 -*-
"""urls"""

from django.urls import path, re_path
from django.contrib.auth.views import LoginView, PasswordChangeView, PasswordResetView, PasswordResetConfirmView

from .forms import BsPasswordChangeForm, BsPasswordResetForm, EmailAuthForm, BsSetPasswordForm


urlpatterns = [
    path(
        'login/',
        LoginView.as_view(authentication_form=EmailAuthForm),
        name='login'
    ),
    path('password_change/',
        PasswordChangeView.as_view(form_class=BsPasswordChangeForm),
        name='password_change'
    ),
    path(
        'password_reset/',
        PasswordResetView.as_view(form_class=BsPasswordResetForm),
        name='password_reset'
    ),
    re_path(
        r'^reset/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$',
        PasswordResetConfirmView.as_view(form_class=BsSetPasswordForm),
        name='password_reset_confirm'
    ),
]



# -*- coding: utf-8 -*-
"""
Contains several optional applications for coop_cms
"""

from django.apps import AppConfig


class CoopCmsAppConfig(AppConfig):
    name = 'coop_cms'
    verbose_name = "coop CMS"
    default_auto_field = 'django.db.models.BigAutoField'



# -*- coding: utf-8 -*-
"""This can be added to default context"""

from django.conf import settings
from django.urls import reverse

from .models import get_homepage_url
from .settings import homepage_no_redirection


def cms_settings(request):
    """add jquery version to context"""
    context = {}

    if hasattr(settings, 'COOP_CMS_JQUERY_VERSION'):
        context['COOP_CMS_JQUERY_VERSION'] = settings.COOP_CMS_JQUERY_VERSION
    
    return context


def homepage_url(request):
    """add hoempage url to context"""
    context = {
        'homepage_url': reverse('coop_cms_homepage') if homepage_no_redirection() else get_homepage_url() ,
    }
    return context



# Generated by Django 4.2.20 on 2026-03-10 08:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('coop_cms', '0021_newsletter_is_visible'),
    ]

    operations = [
        migrations.CreateModel(
            name='DefaultNewsletterSubjectPrefix',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('subject_prefix', models.CharField(max_length=100)),
            ],
            options={
                'verbose_name': 'Préfixe sujet newsletter par défaut',
                'verbose_name_plural': 'Préfixes sujet newsletter par défaut',
            },
        ),
    ]



# -*- coding: utf-8 -*-

from django.db import migrations, models


def set_alias_sites(apps, schema_editor):
    # Associate every alias with all sites
    site_class = apps.get_model("sites", "Site")
    alias_class = apps.get_model("coop_cms", "Alias")

    for alias in alias_class.objects.all():
        for site in site_class.objects.all():
            alias.sites.add(site)
        alias.save()


def reset_alias_sites(apps, schema_editor):
    # Clear the sites for all alias
    alias_class = apps.get_model("coop_cms", "Alias")

    for alias in alias_class.objects.all():
        alias.sites.clear()
        alias.save()


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0001_initial'),
        ('coop_cms', '0009_auto_20170301_1621'),
    ]

    operations = [
        migrations.AddField(
            model_name='alias',
            name='sites',
            field=models.ManyToManyField(to='sites.Site', verbose_name='sites', blank=True),
        ),
        migrations.RunPython(set_alias_sites, reset_alias_sites),
    ]



# Generated by Django 4.2.20 on 2026-03-10 08:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('newsletters', '0004_subscription_unsubscription_reason'),
    ]

    operations = [
        migrations.AddField(
            model_name='subscriptiontype',
            name='subject_prefix',
            field=models.CharField(blank=True, default='', help_text='Éviter de mettre tout en majuscule -> SPAM', max_length=100, verbose_name='préfixe du sujet'),
        ),
    ]



# Generated by Django 4.2.20 on 2025-09-29 12:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('newsletters', '0002_alter_contact_id_alter_emailing_id_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='contact',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='emailing',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='magiclink',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='subscriptiontype',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]



# Generated by Django 4.2.16 on 2024-10-17 08:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('rss_sync', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='rssitem',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='rsssource',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='rsssource',
            name='title',
            field=models.CharField(blank=True, default='', max_length=200, verbose_name='title'),
        ),
    ]



# Generated by Django 4.2.16 on 2024-10-17 08:55

import coop_cms.models
import datetime
from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('coop_cms', '0020_alter_alias_id_alter_articlecategory_id_and_more'),
        ('demo_cms', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='article',
            name='login_required',
            field=models.BooleanField(default=False, help_text='If true, only user with login/password will able to access the article', verbose_name='login required'),
        ),
        migrations.AlterField(
            model_name='article',
            name='category',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='%(app_label)s_%(class)s_rel', to='coop_cms.articlecategory', verbose_name='Category'),
        ),
        migrations.AlterField(
            model_name='article',
            name='content',
            field=models.TextField(blank=True, default='', verbose_name='content'),
        ),
        migrations.AlterField(
            model_name='article',
            name='created',
            field=django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created'),
        ),
        migrations.AlterField(
            model_name='article',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='article',
            name='logo',
            field=models.ImageField(blank=True, default='', null=True, upload_to=coop_cms.models.get_logo_folder),
        ),
        migrations.AlterField(
            model_name='article',
            name='modified',
            field=django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified'),
        ),
        migrations.AlterField(
            model_name='article',
            name='publication_date',
            field=models.DateTimeField(blank=True, default=datetime.datetime.now, verbose_name='Publication date'),
        ),
        migrations.AlterField(
            model_name='article',
            name='sites',
            field=models.ManyToManyField(default=[], to='sites.site', verbose_name='site'),
        ),
        migrations.AlterField(
            model_name='article',
            name='slug',
            field=models.CharField(db_index=True, max_length=100, null=True, unique=True, validators=[coop_cms.models.validate_slug]),
        ),
        migrations.AlterField(
            model_name='article',
            name='subtitle',
            field=models.TextField(blank=True, default='', verbose_name='subtitle'),
        ),
        migrations.AlterField(
            model_name='article',
            name='summary',
            field=models.TextField(blank=True, default='', verbose_name='Summary'),
        ),
        migrations.AlterField(
            model_name='article',
            name='temp_logo',
            field=models.ImageField(blank=True, default='', null=True, upload_to=coop_cms.models.get_logo_folder),
        ),
        migrations.AlterField(
            model_name='article',
            name='template',
            field=models.CharField(blank=True, default='', max_length=200, verbose_name='template'),
        ),
        migrations.AlterField(
            model_name='article',
            name='title',
            field=models.TextField(blank=True, default='', verbose_name='title'),
        ),
    ]



# -*- coding: utf-8 -*-
"""test email sending"""

from django.conf import settings
from django.contrib.sites.models import Site

from ....tests.test_newsletter import NewsletterTest as BaseNewsletterTest


class NewsletterTest(BaseNewsletterTest):
    """test coop_cms newsletter"""

    def test_send_newsletter_template(self):
        def extra_checker(e):
            site = Site.objects.get(id=settings.SITE_ID)
            url = "http://"+site.domain+"/this-link-without-prefix-in-template"
            self.assertTrue(e.alternatives[0][0].find(url) >= 0)
        super(NewsletterTest, self).test_send_test_newsletter('test/newsletter_contact.html')



# -*- coding: utf-8 -*-
"""utilities"""

from datetime import datetime
import re
import sys

from django.conf import settings
from django.core.mail import get_connection, EmailMultiAlternatives
from django.template.loader import get_template
from django.utils import translation
from django.utils.translation import get_language as django_get_language
from django.urls import reverse

from ...logger import logger
from ...models import Newsletter
from ...settings import get_newsletter_context_callbacks
from ...utils import dehtml, make_links_absolute, dehtml_no_uri_count_odd, force_line_max_length

from .settings import get_ignored_magic_links
from .models import Emailing, MagicLink, Contact


class EmailSendError(Exception):
    """An exception raise when sending email failed"""
    pass


def format_context(text, data):
    """replace custom templating by something compliant with python format function"""

    # { and } need to be escaped for the format function
    text = text.replace('{', '{{').replace('}', '}}')

    # #!- and -!# are turned into { and }
    text = text.replace('#!-', '{').replace('-!#', '}')

    return text.format(**data)


def get_emailing_context(emailing, contact):
    """get context for emailing: user,...."""
    data = dict(contact.__dict__)
    for field in ('fullname', ):
        data[field] = getattr(contact, field)
    
    # clone the object: Avoid overwriting {tags} for ever
    newsletter = Newsletter()
    newsletter.__dict__ = dict(emailing.newsletter.__dict__)

    newsletter.subject = format_context(newsletter.subject, data)

    html_content = format_context(newsletter.content, data)

    unregister_url = newsletter.get_site_prefix() + reverse(
        'newsletters:unregister', args=[emailing.id, contact.uuid]
    )

    newsletter.content = html_content
    subject = dehtml(newsletter.subject).replace('\n', '').replace('\r', '')
    if subject.isupper():
        subject = subject.capitalize()
    if emailing.subscription_type.subject_prefix:
        subject = "{0} - {1}".format(emailing.subscription_type.subject_prefix, subject)

    context_dict = {
        'title': subject,
        'newsletter': newsletter,
        'by_email': True,
        'MEDIA_URL': settings.MEDIA_URL,
        'STATIC_URL': settings.STATIC_URL,
        'SITE_PREFIX': emailing.get_domain_url_prefix(),
        'subscription_type_name': emailing.subscription_type.name,
        'unregister_url': unregister_url,
        'contact': contact,
        'emailing': emailing,
    }
    
    for callback in get_newsletter_context_callbacks():
        dictionary = callback(newsletter)
        if dictionary:
            context_dict.update(dictionary)

    return context_dict


def patch_emailing_html(html_text, emailing, contact):
    """transform links into magic link"""
    links = re.findall('href="(?P<url>.+?)"', html_text)

    ignore_links = [
        reverse("newsletters:unregister", args=[emailing.id, contact.uuid]),
        reverse("newsletters:view_online", args=[emailing.id, contact.uuid]),
    ]

    for lang_tuple in settings.LANGUAGES:
        lang = lang_tuple[0][:2]
        ignore_links.append(
            reverse("newsletters:view_online_lang", args=[emailing.id, contact.uuid, lang])
        )

    ignore_links += get_ignored_magic_links()

    # html_text = html_text.replace(
    #     'href="mailto:', 'href="mailto:{0}'.format(settings.COOP_CMS_REPLY_TO)
    # )

    for link in links:
        if (not link.lower().startswith('mailto:')) and (link[0] != "#") and link not in ignore_links:
            # mailto, internal links, 'unregister' and 'view online' are not magic
            if len(link) < 500:

                magic_links = MagicLink.objects.filter(emailing=emailing, url=link)
                if magic_links.count() == 0:
                    magic_link = MagicLink.objects.create(emailing=emailing, url=link)
                else:
                    magic_link = magic_links[0]

                view_magic_link_url = reverse('newsletters:view_link', args=[magic_link.uuid, contact.uuid])
                magic_url = emailing.newsletter.get_site_prefix() + view_magic_link_url
                html_text = html_text.replace('href="{0}"'.format(link), 'href="{0}"'.format(magic_url))
            else:
                if 'test' not in sys.argv:
                    logger.warning(
                        "magic link size is greater than 500 ({0}) : {1}".format(len(link), link)
                    )
    return html_text


def send_newsletter(emailing, max_nb):
    """send newsletter"""

    # Clean the urls
    emailing.newsletter.content = make_links_absolute(
        emailing.newsletter.content, emailing.newsletter, site_prefix=emailing.get_domain_url_prefix()
    )
    
    connection = get_connection()
    from_email = emailing.subscription_type.from_email or settings.COOP_CMS_FROM_EMAIL
    emails = []
    
    contacts = list(emailing.send_to.all()[:max_nb])
    for contact in contacts:
        
        if contact.email:
            lang = emailing.lang or contact.favorite_language or settings.LANGUAGE_CODE[:2]
            translation.activate(lang)

            emailing_context = get_emailing_context(emailing, contact)
            emailing_context["LANGUAGE_CODE"] = lang
            context = emailing_context
            the_template = get_template(emailing.newsletter.get_template_name())

            html_text = the_template.render(context)

            html_text = patch_emailing_html(html_text, emailing, contact)

            html_text = make_links_absolute(
                html_text, emailing.newsletter, site_prefix=emailing.get_domain_url_prefix()
            )
            
            text = dehtml_no_uri_count_odd(html_text)
            list_unsubscribe_url = emailing.get_domain_url_prefix() + reverse(
                "newsletters:unregister", args=[emailing.id, contact.uuid]
            )
            list_unsubscribe_email = getattr(settings, 'COOP_CMS_REPLY_TO', '') or from_email
            headers = {
                "List-Unsubscribe": "<{0}>, <mailto:{1}?subject=unsubscribe>".format(
                    list_unsubscribe_url, list_unsubscribe_email
                )
            }

            if getattr(settings, 'COOP_CMS_REPLY_TO', None):
                headers['Reply-To'] = settings.COOP_CMS_REPLY_TO

            email = EmailMultiAlternatives(
                context['title'],
                force_line_max_length(text),
                from_email,
                [contact.email],
                headers=headers
            )
            html_text = force_line_max_length(html_text, max_length_per_line=400, dont_cut_in_quotes=True)
            email.attach_alternative(html_text, "text/html")
            emails.append(email)

        # print contact, "processed"
        emailing.send_to.remove(contact)
        emailing.sent_to.add(contact)
    
    emailing.save()
    nb_sent = connection.send_messages(emails)
    return nb_sent or 0


def on_bounce(event_type, email, description, permanent, contact_uuid, emailing_id):
    """can be called to signal soft or hard bounce"""
    contacts = Contact.objects.filter(email=email)

    # Unsubscribe emails for permanent errors
    if permanent:
        all_contacts = list(contacts)

        for contact in all_contacts:
            for subscription in contact.subscription_set.all():
                subscription.accept_subscription = False
                subscription.unsubscription_date = datetime.now()
                subscription.save()

    # Update emailing statistics
    if contact_uuid and emailing_id:
        try:
            contact = Contact.objects.get(uuid=contact_uuid)
        except Contact.DoesNotExist:
            contact = None

        try:
            emailing = Emailing.objects.get(id=emailing_id)
        except Emailing.DoesNotExist:
            emailing = None

        if contact and emailing and hasattr(emailing, event_type):
            getattr(emailing, event_type).add(contact)
            emailing.save()


def get_language():
    """wrap the django get_language and make sure: we return 2 chars"""
    lang = django_get_language()
    return lang[:2]



# -*- coding: utf-8 -*-

from django.conf import settings
from django.contrib.auth.models import User, Permission
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django.urls import reverse

from model_mommy import mommy

from colorbox.utils import assert_popup_redirects

from ..models import Link
from ..settings import get_article_class
from ..utils import get_login_url
from . import BaseArticleTest, BeautifulSoup


class AddLinkTest(BaseArticleTest):

    def _log_as_editor(self):
        self.user = user = User.objects.create_user('toto', 'toto@toto.fr', 'toto')

        link_ct = ContentType.objects.get_for_model(Link)
        perm = 'add_{0}'.format(link_ct.model)
        can_add_link = Permission.objects.get(content_type=link_ct, codename=perm)
        user.user_permissions.add(can_add_link)

        user.is_active = True
        user.save()
        return self.client.login(username='toto', password='toto')

    def _log_as_non_editor(self):
        self.user = user = User.objects.create_user('toto', 'toto@toto.fr', 'toto')
        user.save()
        return self.client.login(username='toto', password='toto')

    def test_view_new_link(self):
        url = reverse('coop_cms_new_link')
        self._log_as_editor()
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        self.assertEqual(Link.objects.count(), 0)
        soup = BeautifulSoup(response.content)
        self.assertEqual(1, len(soup.select("input#id_title")))
        self.assertEqual(1, len(soup.select("input#id_url")))
        self.assertEqual(1, len(soup.select("input#id_sites_0")))
        self.assertNotEqual(None, soup.select("input#id_sites_0")[0].get("checked", None))
        self.assertEqual(0, len(soup.select("input#id_sites_1")))

    def test_view_new_link_two_sites(self):
        mommy.make(Site)
        url = reverse('coop_cms_new_link')
        self._log_as_editor()
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        self.assertEqual(Link.objects.count(), 0)
        soup = BeautifulSoup(response.content)
        self.assertEqual(1, len(soup.select("input#id_title")))
        self.assertEqual(1, len(soup.select("input#id_url")))
        self.assertEqual(1, len(soup.select("input#id_sites_0")))
        self.assertEqual(1, len(soup.select("input#id_sites_1")))
        elt1 = soup.select("input#id_sites_0")[0]
        elt2 = soup.select("input#id_sites_1")[0]
        for elt in (elt1, elt2):
            # Only the current site is checked
            self.assertEqual('checked' in elt, elt["value"] == settings.SITE_ID)

    def test_view_new_link_non_editor(self):
        url = reverse('coop_cms_new_link')
        self._log_as_non_editor()
        response = self.client.get(url)
        self.assertEqual(403, response.status_code)

    def test_view_new_link_anonymous(self):
        url = reverse('coop_cms_new_link')
        response = self.client.get(url)
        auth_url = get_login_url()
        self.assertNotAllowed(response)

    def test_add_link(self):
        self._log_as_editor()
        data = {'title': "test", 'url': "https://www.google.fr", 'sites': [settings.SITE_ID]}

        response = self.client.post(reverse('coop_cms_new_link'), data=data)

        assert_popup_redirects(response, reverse('coop_cms_homepage'))

        self.assertEqual(Link.objects.count(), 1)
        link = Link.objects.all()[0]

        self.assertEqual(link.title, data['title'])
        self.assertEqual(link.url, data['url'])
        self.assertEqual(sorted([site.id for site in link.sites.all()]), [settings.SITE_ID])

    def test_add_link_internal(self):
        self._log_as_editor()
        data = {'title': "test", 'url': "/test", 'sites': [settings.SITE_ID]}

        response = self.client.post(reverse('coop_cms_new_link'), data=data)

        assert_popup_redirects(response, reverse('coop_cms_homepage'))

        self.assertEqual(Link.objects.count(), 1)
        link = Link.objects.all()[0]

        self.assertEqual(link.title, data['title'])
        self.assertEqual(link.url, data['url'])
        self.assertEqual(sorted([site.id for site in link.sites.all()]), [settings.SITE_ID])

    def test_add_link_multi_sites(self):
        site2 = mommy.make(Site)
        mommy.make(Site)

        self._log_as_editor()
        data = {'title': "test", 'url': "https://www.google.fr", 'sites': [settings.SITE_ID, site2.id]}

        response = self.client.post(reverse('coop_cms_new_link'), data=data)

        assert_popup_redirects(response, reverse('coop_cms_homepage'))

        self.assertEqual(Link.objects.count(), 1)
        link = Link.objects.all()[0]

        self.assertEqual(link.title, data['title'])
        self.assertEqual(link.url, data['url'])
        self.assertEqual(sorted([site.id for site in link.sites.all()]), [settings.SITE_ID, site2.id])

    def test_add_link_other_site(self):
        site2 = mommy.make(Site)

        self._log_as_editor()
        data = {'title': "test", 'url': "https://www.google.fr", 'sites': [site2.id]}

        response = self.client.post(reverse('coop_cms_new_link'), data=data)

        assert_popup_redirects(response, reverse('coop_cms_homepage'))

        self.assertEqual(Link.objects.count(), 1)
        link = Link.objects.all()[0]

        self.assertEqual(link.title, data['title'])
        self.assertEqual(link.url, data['url'])
        self.assertEqual(sorted([site.id for site in link.sites.all()]), [site2.id])

    def test_add_link_no_site(self):
        site2 = mommy.make(Site)

        self._log_as_editor()
        data = {'title': "test", 'url': "https://www.google.fr", 'sites': []}

        response = self.client.post(reverse('coop_cms_new_link'), data=data)

        assert_popup_redirects(response, reverse('coop_cms_homepage'))

        self.assertEqual(Link.objects.count(), 1)
        link = Link.objects.all()[0]

        self.assertEqual(link.title, data['title'])
        self.assertEqual(link.url, data['url'])
        self.assertEqual(sorted([site.id for site in link.sites.all()]), [])

    def test_add_link_not_allowed(self):
        self._log_as_non_editor()
        data = {'title': "test", 'url': "https://www.google.fr", 'sites': [settings.SITE_ID]}

        response = self.client.post(reverse('coop_cms_new_link'), data=data)

        self.assertEqual(response.status_code, 403)

        self.assertEqual(Link.objects.count(), 0)

    def test_add_link_anonymous(self):
        data = {'title': "test", 'url': "https://www.google.fr", 'sites': [settings.SITE_ID]}
        url = reverse('coop_cms_new_link')
        response = self.client.post(url, data=data)
        self.assertNotAllowed(response)
        self.assertEqual(Link.objects.count(), 0)

    def test_add_link_no_url(self):

        self._log_as_editor()
        data = {'title': "test", 'url': "", 'sites': [settings.SITE_ID]}

        response = self.client.post(reverse('coop_cms_new_link'), data=data, follow=True)
        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select("ul.errorlist")), 1)

        self.assertEqual(Link.objects.count(), 0)

    def test_add_link_no_title(self):
        self._log_as_editor()
        data = {'title': "", 'url': "https://www.google.fr", 'sites': [settings.SITE_ID]}

        response = self.client.post(reverse('coop_cms_new_link'), data=data, follow=True)
        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select("ul.errorlist")), 1)

        self.assertEqual(Link.objects.count(), 0)



# -*- coding: utf-8 -*-
"""forms"""

from django.utils.translation import gettext_lazy as _

import floppyforms as forms

from ...bs_forms import Form as BsForm


class UnregisterForm(BsForm):
    """User wants to unregister from emailing"""
    reason = forms.CharField(required=False, widget=forms.Textarea, label=_("Reason"))



# Generated by Django 2.0.13 on 2019-11-12 13:53

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('coop_cms', '0016_auto_20190509_2158'),
        ('sites', '0002_alter_domain_unique'),
    ]

    operations = [
        migrations.CreateModel(
            name='Contact',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(db_index=True, max_length=254, verbose_name='email')),
                ('last_name', models.CharField(blank=True, default='', max_length=200, verbose_name='last name')),
                ('first_name', models.CharField(blank=True, default='', max_length=200, verbose_name='first name')),
                ('email_verified', models.BooleanField(default=False, verbose_name='email verified')),
                ('uuid', models.CharField(blank=True, db_index=True, default='', max_length=200, verbose_name='unique identifier')),
                ('favorite_language', models.CharField(blank=True, default='', max_length=10, verbose_name='favorite language')),
            ],
            options={
                'verbose_name': 'Contact',
                'verbose_name_plural': 'Contacts',
            },
        ),
        migrations.CreateModel(
            name='Emailing',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.IntegerField(choices=[(1, 'Edition in progress'), (2, 'Sending is scheduled'), (3, 'Sending in progress'), (4, 'Sent')], default=1)),
                ('creation_dt', models.DateTimeField(blank=True, default=None, null=True, verbose_name='creation date')),
                ('scheduling_dt', models.DateTimeField(blank=True, default=None, null=True, verbose_name='scheduling date')),
                ('sending_dt', models.DateTimeField(blank=True, default=None, null=True, verbose_name='sending date')),
                ('newsletter', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='coop_cms.Newsletter')),
                ('opened_emails', models.ManyToManyField(blank=True, related_name='emailing_opened', to='newsletters.Contact')),
                ('send_to', models.ManyToManyField(blank=True, related_name='emailing_to_be_received', to='newsletters.Contact')),
                ('sent_to', models.ManyToManyField(blank=True, related_name='emailing_received', to='newsletters.Contact')),
            ],
            options={
                'verbose_name': 'emailing',
                'verbose_name_plural': 'emailings',
            },
        ),
        migrations.CreateModel(
            name='MagicLink',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.URLField(max_length=500)),
                ('uuid', models.CharField(blank=True, db_index=True, default='', max_length=100)),
                ('emailing', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='newsletters.Emailing')),
                ('visitors', models.ManyToManyField(blank=True, to='newsletters.Contact')),
            ],
            options={
                'verbose_name': 'Magic link',
                'verbose_name_plural': 'Magic links',
            },
        ),
        migrations.CreateModel(
            name='Subscription',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('accept_subscription', models.BooleanField(default=False, help_text='Keep this checked if you want to receive our newsletter.', verbose_name='accept subscription')),
                ('subscription_date', models.DateTimeField(blank=True, default=None, null=True)),
                ('unsubscription_date', models.DateTimeField(blank=True, default=None, null=True)),
                ('contact', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='newsletters.Contact')),
            ],
        ),
        migrations.CreateModel(
            name='SubscriptionType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, verbose_name='name')),
                ('order_index', models.IntegerField(default=0, verbose_name='order index')),
                ('from_email', models.CharField(blank=True, default='', max_length=100, verbose_name='From email')),
                ('lang', models.CharField(blank=True, default='', max_length=5, verbose_name='language')),
                ('allowed_on_sites', models.ManyToManyField(blank=True, related_name='_subscriptiontype_allowed_on_sites_+', to='sites.Site')),
                ('site', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.Site', related_name='+')),
            ],
            options={
                'verbose_name': 'Subscription type',
                'verbose_name_plural': 'Subscription types',
                'ordering': ('order_index',),
            },
        ),
        migrations.AddField(
            model_name='subscription',
            name='subscription_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='newsletters.SubscriptionType'),
        ),
        migrations.AddField(
            model_name='emailing',
            name='subscription_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='newsletters.SubscriptionType'),
        ),
        migrations.AddField(
            model_name='emailing',
            name='unsub',
            field=models.ManyToManyField(blank=True, related_name='emailing_unsub', to='newsletters.Contact'),
        ),
    ]



# -*- coding: utf-8 -*-

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('coop_cms', '0008_alias_redirect_code'),
    ]

    operations = [
        migrations.AddField(
            model_name='sitesettings',
            name='homepage_article',
            field=models.CharField(default='', max_length=256, blank=True, help_text='if set, the homepage will get the article with the given slug', verbose_name='homepage article', db_index=True),
        ),
        migrations.AlterField(
            model_name='sitesettings',
            name='homepage_url',
            field=models.CharField(default='', max_length=256, blank=True, help_text='if set, the homepage will be redirected to the given URL', verbose_name='homepage URL', db_index=True),
        ),
    ]



# -*- coding: utf-8 -*-

"""
modeltranslation config file : which field to be translatable
"""

from modeltranslation.translator import translator, TranslationOptions  # pylint: disable=F0401

from .models import Article


class ArticleTranslationOptions(TranslationOptions):
    """Translate article"""
    fields = ('slug', 'title', 'content', 'summary', 'subtitle')


translator.register(Article, ArticleTranslationOptions)



# -*- coding: utf-8 -*-
"""
Email authentication Unit tests
"""

from unittest import skipUnless

from django.conf import settings
from django.contrib.auth.models import User
from django.core import mail, management
from django.test import TestCase
from django.test.utils import override_settings
from django.urls import reverse

from model_mommy import mommy

from ...moves import is_anonymous
from ...tests import BeautifulSoup

from .models import InvalidatedUser


TEST_AUTHENTICATION_BACKENDS = (
    'coop_cms.perms_backends.ArticlePermissionBackend',
    'coop_cms.apps.email_auth.auth_backends.EmailAuthBackend',
    'django.contrib.auth.backends.ModelBackend',  # Django's default auth backend
)


class BaseTest(TestCase):
    """Base class for TestCase"""

    def _make(self, klass, **kwargs):
        """Make an object"""
        password = None
        if klass == User:
            password = kwargs.pop('password', None)
        obj = mommy.make(klass, **kwargs)
        if password:
            obj.set_password(password)
            obj.save()
        return obj


@override_settings(AUTHENTICATION_BACKENDS=TEST_AUTHENTICATION_BACKENDS)
class EmailAuthBackendTest(BaseTest):
    """Email auth test case"""

    def test_email_login(self):
        """Test user can login with email"""
        user = self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")
        login_ok = self.client.login(email=user.email, password="password")
        self.assertEqual(login_ok, True)

    def test_email_login_inactve(self):
        """Test user can not login if inactive"""
        user = self._make(User, is_active=False, password="password", email="toto@toto.fr", username="toto")
        login_ok = self.client.login(email=user.email, password="password")
        self.assertEqual(login_ok, False)

    def test_email_login_not_exists(self):
        """Test can not login if email does'nt exist"""
        login_ok = self.client.login(email="titi@titi.fr", password="password")
        self.assertEqual(login_ok, False)

    def test_email_login_several(self):
        """test can login if several user with same email"""
        user1 = self._make(User, is_active=True, password="password1", email="toto@toto.fr", username="toto1")
        user2 = self._make(User, is_active=True, password="password2", email="toto@toto.fr", username="toto2")
        login_ok = self.client.login(email=user1.email, password="password1")
        self.assertEqual(login_ok, True)
        self.client.logout()
        login_ok = self.client.login(email=user2.email, password="password2")
        self.assertEqual(login_ok, True)

    def test_email_login_several_one_inactive(self):
        """test user can login if several user with email and one is inactive"""
        user1 = self._make(User, is_active=False, password="password1", email="toto@toto.fr", username="toto1")
        user2 = self._make(User, is_active=False, password="password2", email="toto@toto.fr", username="toto2")
        login_ok = self.client.login(email=user1.email, password="password1")
        self.assertEqual(login_ok, False)
        self.client.logout()
        login_ok = self.client.login(email=user2.email, password="password2")
        self.assertEqual(login_ok, False)

    def test_email_login_several_all_inactive(self):
        """test user can not login if several user with email but all are inactive"""
        user1 = self._make(User, is_active=False, password="password1", email="toto@toto.fr", username="toto1")
        user2 = self._make(User, is_active=True, password="password2", email="toto@toto.fr", username="toto2")
        login_ok = self.client.login(email=user1.email, password="password1")
        self.assertEqual(login_ok, False)
        self.client.logout()
        login_ok = self.client.login(email=user2.email, password="password2")
        self.assertEqual(login_ok, True)

    def test_email_login_wrong_password(self):
        """test user can not login if wrong password"""
        user = self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")
        login_ok = self.client.login(email=user.email, password="toto")
        self.assertEqual(login_ok, False)


@override_settings(AUTHENTICATION_BACKENDS=TEST_AUTHENTICATION_BACKENDS)
class UserLoginTest(BaseTest):
    """Test the login page"""

    def test_view_login(self):
        """test the user can view the login page"""
        url = reverse("login")
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        self.assertTrue(len(soup.select("input[name=email]")) > 0)
        self.assertTrue(len(soup.select("input[name=password]")) > 0)
        self.assertEqual(0, len(soup.select("input[name=username]")))

    def test_post_login(self):
        """test the user can login from login page"""
        user = self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")
        url = reverse("login")

        data = {
            'password': 'password',
            'email': user.email,
        }

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 302)

        user_id = self.client.session.get("_auth_user_id", 0)
        self.assertEqual(int(user_id), user.id)

    def test_post_login_wrong_password(self):
        """test error if user login from the login page: wrong password"""
        user = self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")

        url = reverse("login")

        data = {
            'password': 'password&&',
            'email': user.email,
        }

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        user_id = self.client.session.get("_auth_user_id", 0)
        self.assertEqual(user_id, 0)

    def test_post_login_wrong_email(self):
        """test error if user login from the login page: wrong email"""

        self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")

        url = reverse("login")

        data = {
            'password': 'password',
            'email': 'toto@toto.com',
        }

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        user_id = self.client.session.get("_auth_user_id", 0)
        self.assertEqual(user_id, 0)

    def test_post_login_invalid_email(self):
        """test error if user login from the login page: invalid email"""

        self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")

        url = reverse("login")

        data = {
            'password': 'password',
            'email': "a",
        }

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        user_id = self.client.session.get("_auth_user_id", 0)
        self.assertEqual(user_id, 0)

    def test_post_login_missing_password(self):
        """test error if user login from the login page: password is missing"""

        user = self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")

        url = reverse("login")

        data = {
            'password': '',
            'email': user.email,
        }

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        user_id = self.client.session.get("_auth_user_id", 0)
        self.assertEqual(user_id, 0)

    def test_post_login_missing_email(self):
        """test error if user login from the login page: email is missing"""

        self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")

        url = reverse("login")

        data = {
            'password': 'password',
            'email': '',
        }

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        user_id = self.client.session.get("_auth_user_id", 0)
        self.assertEqual(user_id, 0)

    def test_post_login_missing_both(self):
        """test error if user login from the login page: user and password are missing"""

        self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")

        url = reverse("login")

        data = {
            'password': '',
            'email': "",
        }

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        user_id = self.client.session.get("_auth_user_id", 0)
        self.assertEqual(user_id, 0)

    def test_post_login_inactive_user(self):
        """test error if user login from the login page: inactive user"""

        user = self._make(User, is_active=False, password="password", email="toto@toto.fr", username="toto")

        url = reverse("login")

        data = {
            'password': 'password',
            'email': user.email,
        }
        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        user_id = self.client.session.get("_auth_user_id", 0)
        self.assertEqual(user_id, 0)

    def test_post_login_username(self):
        """test error if user login from the login page: login with username"""

        user = self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")

        url = reverse("login")

        data = {
            'password': 'password',
            'username': user.username,
        }

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        user_id = self.client.session.get("_auth_user_id", 0)
        self.assertEqual(user_id, 0)


@skipUnless('django_registration' in settings.INSTALLED_APPS, "django_registration not installed installed")
@override_settings(AUTHENTICATION_BACKENDS=TEST_AUTHENTICATION_BACKENDS)
class RegistrationTest(BaseTest):
    """Test that events are sent on registration events"""

    def test_view_register(self):
        """It should display form"""
        user_count = User.objects.count()
        url = reverse('registration_register')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(User.objects.count(), user_count)
        self.assertEqual(len(mail.outbox), 0)

        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select("input#id_email")), 1)
        self.assertEqual(len(soup.select("input#id_password1")), 1)
        self.assertEqual(len(soup.select("input#id_password2")), 1)
        self.assertEqual(len(soup.select("input#id_username")), 1)
        self.assertEqual(len(soup.select("input#id_terms_of_service")), 1)

        self.assertEqual(soup.select("input#id_email")[0]['type'], 'email')
        self.assertEqual(soup.select("input#id_password1")[0]['type'], 'password')
        self.assertEqual(soup.select("input#id_password2")[0]['type'], 'password')
        self.assertEqual(soup.select("input#id_username")[0]['type'], 'hidden')
        self.assertEqual(soup.select("input#id_terms_of_service")[0]['type'], 'checkbox')

    def test_register(self):
        """It should create disabled user"""
        url = reverse('registration_register')
        data = {
            'email': 'john.doe@company.com',
            'password1': 'blabla-123',
            'password2': 'blabla-123',
            'terms_of_service': True,
        }
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(User.objects.filter(email=data['email']).count(), 1)
        user = User.objects.filter(email=data['email'])[0]
        self.assertEqual(user.is_active, False)
        self.assertEqual(user.is_staff, False)
        self.assertEqual(user.is_superuser, False)

        self.assertEqual(len(mail.outbox), 1)
        email = mail.outbox[0]
        self.assertEqual(email.to, [data['email']])

    def test_register_refuse_terms(self):
        """It should display error"""
        url = reverse('registration_register')
        data = {
            'email': 'john.doe@company.com',
            'password1': 'blabla-123',
            'password2': 'blabla-123',
            'terms_of_service': False,
        }
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select('.errorlist')), 1)
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)
        self.assertEqual(len(mail.outbox), 0)

    def test_register_refuse_wrong_password(self):
        """It should display error"""
        url = reverse('registration_register')
        data = {
            'email': 'john.doe@company.com',
            'password1': 'blabla-123',
            'password2': 'blabla-124',
            'terms_of_service': True,
        }
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select('.errorlist')), 1)
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)
        self.assertEqual(len(mail.outbox), 0)

    def test_register_refuse_no_password(self):
        """It should display error"""
        url = reverse('registration_register')
        data = {
            'email': 'john.doe@company.com',
            'password1': '',
            'password2': '',
            'terms_of_service': True,
        }
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select('.errorlist')), 2)
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)
        self.assertEqual(len(mail.outbox), 0)

    def test_register_refuse_wrong_email(self):
        """It should display error"""
        url = reverse('registration_register')
        data = {
            'email': 'john',
            'password1': 'blabla-123',
            'password2': 'blabla-123',
            'terms_of_service': True,
        }
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select('.errorlist')), 1)
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)
        self.assertEqual(len(mail.outbox), 0)

    @override_settings(COOP_CMS_ACCOUNT_REGISTRATION_NOTIFICATION_EMAILS=['toto@toto.fr'])
    def test_register_notify(self):
        """It should create disabled user"""
        url = reverse('registration_register')
        data = {
            'email': 'john.doe@company.com',
            'password1': 'blabla-123',
            'password2': 'blabla-123',
            'terms_of_service': True,
        }
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(User.objects.filter(email=data['email']).count(), 1)
        user = User.objects.filter(email=data['email'])[0]
        self.assertEqual(user.is_active, False)
        self.assertEqual(user.is_staff, False)
        self.assertEqual(user.is_superuser, False)

        self.assertEqual(len(mail.outbox), 2)
        email = mail.outbox[0]
        self.assertEqual(email.to, [data['email']])
        email = mail.outbox[1]
        self.assertEqual(email.to, ['toto@toto.fr'])
        self.assertTrue(email.body.find(data['email']) > 0)

    def test_activate_user(self):
        """It should activate the user"""
        url = reverse('registration_register')
        data = {
            'email': 'john.doe@company.com',
            'password1': 'blabla-123',
            'password2': 'blabla-123',
            'terms_of_service': True,
        }
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 302)
        activation_key = response.context["activation_key"]

        self.assertEqual(User.objects.filter(email=data['email']).count(), 1)
        user = User.objects.filter(email=data['email'])[0]
        self.assertEqual(user.is_active, False)
        self.assertEqual(user.is_staff, False)
        self.assertEqual(user.is_superuser, False)

        self.assertEqual(len(mail.outbox), 1)
        email = mail.outbox[0]
        self.assertEqual(email.to, [data['email']])
        mail.outbox = []

        activation_url = reverse('registration_activate', args=[activation_key])
        response = self.client.get(activation_url, follow=True)
        self.assertEqual(response.status_code, 200)
        user = User.objects.get(id=user.id)
        self.assertEqual(user.is_active, True)
        self.assertEqual(user.is_staff, False)
        self.assertEqual(user.is_superuser, False)
        self.assertEqual(len(mail.outbox), 0)

    @override_settings(COOP_CMS_ACCOUNT_ACTION_NOTIFICATION_EMAILS=['toto@toto.fr', 'titi@titi.fr'])
    def test_activate_user_and_notify(self):
        """It should activate the user"""
        url = reverse('registration_register')
        data = {
            'email': 'john.doe@company.com',
            'password1': 'blabla-123',
            'password2': 'blabla-123',
            'terms_of_service': True,
        }
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 302)
        activation_key = response.context["activation_key"]

        self.assertEqual(User.objects.filter(email=data['email']).count(), 1)
        user = User.objects.filter(email=data['email'])[0]
        self.assertEqual(user.is_active, False)
        self.assertEqual(user.is_staff, False)
        self.assertEqual(user.is_superuser, False)

        self.assertEqual(len(mail.outbox), 1)
        email = mail.outbox[0]
        self.assertEqual(email.to, [data['email']])
        mail.outbox = []

        activation_url = reverse('registration_activate', args=[activation_key])
        response = self.client.get(activation_url, follow=True)
        self.assertEqual(response.status_code, 200)
        user = User.objects.get(id=user.id)
        self.assertEqual(user.is_active, True)
        self.assertEqual(user.is_staff, False)
        self.assertEqual(user.is_superuser, False)

        self.assertEqual(len(mail.outbox), 1)
        email = mail.outbox[0]
        self.assertEqual(email.to, ['toto@toto.fr', 'titi@titi.fr'])
        self.assertTrue(email.body.find(data['email']) > 0)


@override_settings(AUTHENTICATION_BACKENDS=TEST_AUTHENTICATION_BACKENDS)
class InvalidationUserBackendTest(BaseTest):
    """check invalidated user management"""

    def test_login_user_wrong_password(self):
        """Test what happen when user with invalidated login try to login"""
        user = self._make(User, is_active=True, password="abd134", email="toto@toto.fr", username="toto")
        mommy.make(InvalidatedUser, user=user, password_changed=False)
        login_ok = self.client.login(email=user.email, password="password")
        self.assertEqual(login_ok, False)
        self.assertEqual(InvalidatedUser.objects.count(), 1)
        self.assertEqual(InvalidatedUser.objects.filter(user=user, password_changed=False).count(), 1)

    def test_login(self):
        """Test user can login"""
        user = self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")
        mommy.make(InvalidatedUser, user=user, password_changed=False)
        login_ok = self.client.login(email=user.email, password="password")
        self.assertEqual(login_ok, True)
        self.assertEqual(InvalidatedUser.objects.count(), 1)
        self.assertEqual(InvalidatedUser.objects.filter(user=user, password_changed=False).count(), 0)
        self.assertEqual(InvalidatedUser.objects.filter(user=user, password_changed=True).count(), 1)

    def test_post_login_error_invalidated(self):
        """Test what happen when user with invalidated login try to login"""
        user = self._make(User, is_active=True, password="abd134", email="toto@toto.fr", username="toto")
        mommy.make(InvalidatedUser, user=user, password_changed=False)

        url = reverse('login')

        response = self.client.post(url, dict(email=user.email, password="password"))
        self.assertEqual(response.status_code, 200)

        form = self._get_from_context(response, 'form')
        self.assertNotEqual(form, None)
        self.assertEqual(form.invalidated_password, True)

        self.assertEqual(is_anonymous(self._get_user_from_context(response)), True)

        self.assertEqual(InvalidatedUser.objects.count(), 1)
        self.assertEqual(InvalidatedUser.objects.filter(user=user, password_changed=False).count(), 1)

    def _get_from_context(self, response, var_name):
        """extract variable from context"""
        for context in response.context:
            if var_name in context:
                return context[var_name]

    def _get_user_from_context(self, response):
        """extract variable from context"""
        user = self._get_from_context(response, 'user')
        if user and user.is_authenticated:
            return User.objects.get(id=user.id)
        return user

    def test_post_login_error_not_invalidated(self):
        """Test what happen when user without invalidated login try to login"""
        user = self._make(User, is_active=True, password="abd134", email="toto@toto.fr", username="toto")

        url = reverse('login')

        response = self.client.post(url, dict(email=user.email, password="password"))
        self.assertEqual(response.status_code, 200)

        form = self._get_from_context(response, 'form')
        self.assertNotEqual(form, None)
        self.assertEqual(form.invalidated_password, False)

        self.assertEqual(is_anonymous(self._get_user_from_context(response)), True)

        self.assertEqual(InvalidatedUser.objects.count(), 0)

    def test_post_login_invalidated(self):
        """Test what happen when user with invalidated login, login"""
        user = self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")
        mommy.make(InvalidatedUser, user=user, password_changed=False)

        url = reverse('login')

        response = self.client.post(url, dict(email=user.email, password="password"), follow=True)
        self.assertEqual(response.status_code, 200)

        self.assertEqual(self._get_user_from_context(response), user)

        self.assertEqual(InvalidatedUser.objects.count(), 1)
        self.assertEqual(InvalidatedUser.objects.filter(user=user, password_changed=False).count(), 0)
        self.assertEqual(InvalidatedUser.objects.filter(user=user, password_changed=True).count(), 1)

    def test_post_login_not_invalidated(self):
        """Test what happen when user login"""
        user = self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")

        url = reverse('login')

        response = self.client.post(url, dict(email=user.email, password="password"), follow=True)
        self.assertEqual(response.status_code, 200)

        self.assertEqual(self._get_user_from_context(response), user)

        self.assertEqual(InvalidatedUser.objects.count(), 0)

    def test_create_article_commands(self):
        user1 = self._make(User, is_active=True, password="password", email="toto1@toto.fr", username="toto1")
        user2 = self._make(User, is_active=True, password="password", email="toto2@toto.fr", username="toto2")

        self.assertEqual(InvalidatedUser.objects.count(), 0)

        management.call_command('invalidate_passwords', verbosity=0)

        self.assertEqual(User.objects.count(), 2)
        self.assertEqual(InvalidatedUser.objects.count(), 2)
        for user_id in (user1.id, user2.id):
            user = User.objects.get(id=user_id)
            self.assertEqual(InvalidatedUser.objects.filter(user=user).count(), 1)
            self.assertEqual(InvalidatedUser.objects.filter(user=user, password_changed=False).count(), 1)
            self.assertEqual(user.check_password("password"), False)



# -*- coding: utf-8 -*-

from django.template import Template, Context
from django.test.utils import override_settings

from model_mommy import mommy

from ..models import PieceOfHtml
from ..settings import get_article_class
from . import BaseTestCase, BeautifulSoup, BaseArticleTest


class PieceOfHtmlTagsTest(BaseTestCase):
    
    def test_create_poc(self):
        tpl = Template('{% load coop_edition %}{% coop_piece_of_html "test" %}')
        html = tpl.render(Context({}))
        self.assertEqual(html, "")
        self.assertEqual(PieceOfHtml.objects.count(), 1)
        poc = PieceOfHtml.objects.all()[0]
        self.assertEqual(poc.div_id, "test")
        
    def test_existing_poc(self):
        poc = mommy.make(PieceOfHtml, div_id="test", content="HELLO!!!")
        tpl = Template('{% load coop_edition %}{% coop_piece_of_html "test" %}')
        html = tpl.render(Context({}))
        self.assertEqual(html, poc.content)
        self.assertEqual(PieceOfHtml.objects.count(), 1)
        poc = PieceOfHtml.objects.all()[0]
        self.assertEqual(poc.div_id, "test")
        
    def test_create_poc_read_only(self):
        poc = mommy.make(PieceOfHtml, div_id="test", content="HELLO!!!")
        tpl = Template('{% load coop_edition %}{% coop_piece_of_html "test" read-only %}')
        html = tpl.render(Context({}))
        self.assertEqual(html, poc.content)
        self.assertEqual(PieceOfHtml.objects.count(), 1)
        poc = PieceOfHtml.objects.all()[0]
        self.assertEqual(poc.div_id, "test")
        
    def test_create_edit_poc(self):
        tpl = Template('{% load coop_edition %}{% coop_piece_of_html "test" %}')
        html = tpl.render(Context({"inline_html_edit": True}))
        self.assertNotEqual(html, "")
        
        soup = BeautifulSoup(html)
        tags = soup.select("#html_editor_html_editor__coop_cms__PieceOfHtml__div_id__test__content")
        self.assertEqual(len(tags), 1)
        self.assertEqual(tags[0].text, "")
        
        tags_hidden = soup.select("#html_editor_html_editor__coop_cms__PieceOfHtml__div_id__test__content_hidden")
        self.assertEqual(len(tags_hidden), 1)
        self.assertEqual(tags_hidden[0].get("value", ""), "")
        
        self.assertEqual(PieceOfHtml.objects.count(), 1)
        poc = PieceOfHtml.objects.all()[0]
        self.assertEqual(poc.div_id, "test")
        
    def test_edit_poc(self):
        poc = mommy.make(PieceOfHtml, div_id="test", content="HELLO!!!")
        tpl = Template('{% load coop_edition %}{% coop_piece_of_html "test" %}')
        html = tpl.render(Context({"inline_html_edit": True}))
        self.assertNotEqual(html, poc.content)
        
        soup = BeautifulSoup(html)
        tags = soup.select("#html_editor_html_editor__coop_cms__PieceOfHtml__div_id__test__content")
        self.assertEqual(len(tags), 1)
        self.assertEqual(tags[0].text, poc.content)
        
        tags_hidden = soup.select("#html_editor_html_editor__coop_cms__PieceOfHtml__div_id__test__content_hidden")
        self.assertEqual(len(tags_hidden), 1)
        self.assertEqual(tags_hidden[0]["value"], poc.content)
        
        self.assertEqual(PieceOfHtml.objects.count(), 1)
        poc = PieceOfHtml.objects.all()[0]
        self.assertEqual(poc.div_id, "test")
        
    def test_edit_poc_read_only(self):
        poc = mommy.make(PieceOfHtml, div_id="test", content="HELLO!!!")
        tpl = Template('{% load coop_edition %}{% coop_piece_of_html "test" read-only %}')
        html = tpl.render(Context({"inline_html_edit": True}))
        self.assertEqual(html, poc.content)
        self.assertEqual(PieceOfHtml.objects.count(), 1)
        poc = PieceOfHtml.objects.all()[0]
        self.assertEqual(poc.div_id, "test")
        
    def test_view_poc_extra_id(self):
        poc = mommy.make(PieceOfHtml, div_id="test", content="HELLO!!!", extra_id="1")
        tpl = Template('{% load coop_edition %}{% coop_piece_of_html "test" extra_id=1 %}')
        html = tpl.render(Context({"inline_html_edit": False}))
        self.assertEqual(html, poc.content)
        self.assertEqual(PieceOfHtml.objects.count(), 1)
        poc = PieceOfHtml.objects.all()[0]
        self.assertEqual(poc.div_id, "test")
        self.assertEqual(poc.extra_id, "1")
        
    def test_edit_poc_extra_id(self):
        poc = mommy.make(PieceOfHtml, div_id="test", content="HELLO!!!", extra_id="1")
        tpl = Template('{% load coop_edition %}{% coop_piece_of_html "test" extra_id=1 %}')
        html = tpl.render(Context({"inline_html_edit": True}))
        
        soup = BeautifulSoup(html)
        #print html
        tags = soup.select("input[type=hidden]")
        self.assertEqual(len(tags), 1)
        div_selector = tags[0].attrs['id']
        div_selector = div_selector.replace("_hidden", "")
        
        tags = soup.select("#"+div_selector)
        self.assertEqual(len(tags), 1)
        self.assertEqual(tags[0].text, poc.content)
        
        self.assertEqual(PieceOfHtml.objects.count(), 1)
        poc = PieceOfHtml.objects.all()[0]
        self.assertEqual(poc.div_id, "test")
        self.assertEqual(poc.extra_id, "1")
        
    def test_create_poc_extra_id(self):
        tpl = Template('{% load coop_edition %}{% coop_piece_of_html "test" extra_id=1 %}')
        html = tpl.render(Context({"inline_html_edit": False}))
        self.assertEqual(html, "")
        self.assertEqual(PieceOfHtml.objects.count(), 1)
        poc = PieceOfHtml.objects.all()[0]
        self.assertEqual(poc.div_id, "test")
        self.assertEqual(poc.extra_id, "1")
        
    def test_create_new_poc_extra_id(self):
        poc = mommy.make(PieceOfHtml, div_id="test", content="HELLO!!!", extra_id="1")
        tpl = Template('{% load coop_edition %}{% coop_piece_of_html "test" extra_id=2 %}')
        html = tpl.render(Context({"inline_html_edit": False}))
        self.assertEqual(html, "")
        self.assertEqual(PieceOfHtml.objects.count(), 2)
        PieceOfHtml.objects.get(div_id="test", extra_id="1")
        PieceOfHtml.objects.get(div_id="test", extra_id="2")
        
    def test_poc_extra_id_readonly(self):
        poc = mommy.make(PieceOfHtml, div_id="test", content="HELLO!!!", extra_id="1")
        tpl = Template('{% load coop_edition %}{% coop_piece_of_html "test" read-only extra_id=1 %}')
        html = tpl.render(Context({"inline_html_edit": True}))
        self.assertEqual(html, poc.content)
        self.assertEqual(PieceOfHtml.objects.count(), 1)
        PieceOfHtml.objects.get(div_id="test", extra_id="1")


@override_settings(COOP_CMS_ARTICLE_TEMPLATES=(('test/article_with_blocks.html', 'Article with blocks'),))
class BlockInheritanceTest(BaseArticleTest):
    """test using block templatetag inside the cms_edit template tag"""

    def test_view_with_blocks(self):
        """test view article with block templatetag inside the cms_edit template tag"""

        article_class = get_article_class()
        article = mommy.make(
            article_class,
            title="This is my article", content="<p>This is my <b>content</b></p>",
            template='test/article_with_blocks.html'
        )

        response = self.client.get(article.get_absolute_url())
        self.assertEqual(response.status_code, 200)

        self.assertContains(response, article.title)
        self.assertContains(response, article.content)

        self.assertContains(response, "*** HELLO FROM CHILD ***")
        self.assertContains(response, "*** HELLO FROM PARENT ***")
        self.assertContains(response, "*** HELLO FROM BLOCK ***")

    def test_edit_with_blocks(self):
        """test edition with block templatetag inside the cms_edit template tag"""

        article_class = get_article_class()
        article = mommy.make(
            article_class,
            title="This is my article", content="<p>This is my <b>content</b></p>",
            template='test/article_with_blocks.html'
        )

        self._log_as_editor()

        data = {
            "title": "This is a new title",
            'content': "<p>This is a <i>*** NEW ***</i> <b>content</b></p>"
        }
        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)

        article = article_class.objects.get(id=article.id)

        self.assertEqual(article.title, data['title'])
        self.assertEqual(article.content, data['content'])

        self.assertContains(response, article.title)
        self.assertContains(response, article.content)

        self.assertContains(response, "*** HELLO FROM CHILD ***")
        self.assertContains(response, "*** HELLO FROM PARENT ***")
        self.assertContains(response, "*** HELLO FROM BLOCK ***")




# -*- coding:utf-8 -*-
"""
urls
"""

from django.urls import path

from . import views

urlpatterns = [
    path('collect-rss-items/<int:source_id>/', views.collect_rss_items_view, name='rss_sync_collect_rss_items'),
    path('create-cms-article/<int:item_id>/', views.create_cms_article_view, name='rss_sync_create_cms_article'),
]



# -*- coding: utf-8 -*-
"""
Utilities for Bootstrap CSS framework
"""

from django.apps import AppConfig


class CoopBootstrapAppConfig(AppConfig):
    name = 'coop_cms.apps.coop_bootstrap'
    verbose_name = "coop CMS > Support of Bootstrap CSS framework"



# -*- coding: utf-8 -*-
"""admin site"""

from django.contrib import admin

import floppyforms as forms

from . import models
from . import settings


@admin.register(models.Emailing)
class EmailingAdmin(admin.ModelAdmin):
    """Emailing"""
    list_display = ['newsletter', 'creation_dt', 'subscription_type', 'status']
    list_editable = ['status']
    raw_id_fields = [
        'send_to', 'sent_to', 'opened_emails', 'unsub',
    ]
    search_fields = ['newsletter__subject', 'newsletter__content']
    list_filter = ['status', 'subscription_type']
    date_hierarchy = 'creation_dt'


@admin.register(models.MagicLink)
class MagicLinkAdmin(admin.ModelAdmin):
    """Magic link"""
    list_display = ['url', 'emailing']
    search_fields = ['url', 'emailing']
    raw_id_admin = ('emailing',)


@admin.register(models.SubscriptionType)
class SubscriptionTypeAdmin(admin.ModelAdmin):
    list_display = ('name', 'site', 'order_index', )

    def get_form(self, *args, **kwargs):
        form_class = super().get_form(*args, **kwargs)

        class custom_form_class(form_class):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)
                self.fields['lang'].widget = forms.Select(choices=settings.get_language_choices())

        return custom_form_class


@admin.register(models.Subscription)
class SubscriptionAdmin(admin.ModelAdmin):
    list_display = ('subscription_type', 'contact', )
    raw_id_fields = ('subscription_type', 'contact', )


@admin.register(models.Contact)
class ContactAdmin(admin.ModelAdmin):
    list_display = ('email', 'first_name', 'last_name', 'email_verified')



# Generated by Django 4.2.8 on 2024-01-05 15:01

import datetime
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('coop_cms', '0020_alter_alias_id_alter_articlecategory_id_and_more'),
        ('basic_cms', '0005_auto_20190719_1408'),
    ]

    operations = [
        migrations.AlterField(
            model_name='article',
            name='category',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='%(app_label)s_%(class)s_rel', to='coop_cms.articlecategory', verbose_name='Category'),
        ),
        migrations.AlterField(
            model_name='article',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='article',
            name='publication_date',
            field=models.DateTimeField(blank=True, default=datetime.datetime.now, verbose_name='Publication date'),
        ),
        migrations.AlterField(
            model_name='navtree',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='navtree',
            name='types',
            field=models.ManyToManyField(blank=True, related_name='%(app_label)s_%(class)s_set', to='coop_cms.navtype'),
        ),
    ]



# -*- coding: utf-8 -*-

from getpass import getpass

from django.core.management.base import BaseCommand
from django.conf import settings

from ...secrets import set_db_password


class Command(BaseCommand):
    help = 'Generates a password DB'

    def handle(self, *args, **options):

        db_password = getpass("password?")

        set_db_password(settings.BASE_DIR, settings.SECRET_KEY, db_password)



# Generated by Django 4.2.20 on 2025-09-29 15:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('coop_cms', '0020_alter_alias_id_alter_articlecategory_id_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='newsletter',
            name='is_visible',
            field=models.BooleanField(default=True, verbose_name='visible'),
        ),
    ]



# -*- coding: utf-8 -*-
"""test email sending"""

from unittest import skipIf

from django.conf import settings
from django.test.utils import override_settings
from django.urls import reverse

from model_mommy import mommy

from ....models import Newsletter
from ....settings import is_localized, is_multilang
from ....tests import BaseTestCase


from .. import models


class ViewOnlineTest(BaseTestCase):

    def test_view_online(self):
        contact = mommy.make(models.Contact, email='toto@toto.fr')
        newsletter_data = {
            'subject': 'This is the subject',
            'content': '<h2>Hello #!-fullname-!#!</h2><p>Visit <a href="http://toto.fr">us</a></p>',
            'template': 'test/newsletter_contact.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        emailing = mommy.make(models.Emailing, newsletter=newsletter)
        emailing.sent_to.add(contact)
        emailing.save()
        url = reverse('newsletters:view_online', args=[emailing.id, contact.uuid])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, contact.fullname)
        self.assertEqual(models.MagicLink.objects.count(), 2)
        magic_link0 = models.MagicLink.objects.all()[0]
        self.assertContains(response, reverse('newsletters:view_link', args=[magic_link0.uuid, contact.uuid]))
        self.assertEqual(magic_link0.url, '/this-link-without-prefix-in-template')
        magic_link1 = models.MagicLink.objects.all()[1]
        self.assertContains(response, reverse('newsletters:view_link', args=[magic_link1.uuid, contact.uuid]))
        self.assertEqual(magic_link1.url, 'http://toto.fr')

    @skipIf(not is_localized() or not is_multilang(), "not localized")
    def test_emailing_view_online_lang(self):
        """test view emailing in other lang"""
        contact = mommy.make(models.Contact, email='toto@toto.fr')
        newsletter_data = {
            'subject': 'This is the subject',
            'content': '<h2>Hello #!-fullname-!#!</h2><p>Visit <a href="http://toto.fr">us</a></p>',
            'template': 'test/newsletter_contact.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        emailing = mommy.make(models.Emailing, newsletter=newsletter)
        emailing.sent_to.add(contact)
        emailing.save()
        other_lang = settings.LANGUAGES[1][0]
        url = reverse('newsletters:view_online_lang', args=[emailing.id, contact.uuid, other_lang])
        response = self.client.get(url)
        self.assertEqual(302, response.status_code)
        next_url = reverse('newsletters:view_online', args=[emailing.id, contact.uuid])[3:]
        redirect_url = other_lang + next_url
        self.assertTrue(response['Location'].find(redirect_url) >= 0)

    @override_settings(SECRET_KEY="super-héros")
    def test_view_online_utf_links(self):
        contact = mommy.make(models.Contact, email='toto@toto.fr', first_name='Emmet', last_name='Brown')
        newsletter_data = {
            'subject': 'This is the subject',
            'content': '<h2>Hello #!-fullname-!#!</h2><p>Visit <a href="http://toto.fr/à-bientôt">à bientôt</a></p>',
            'template': 'test/newsletter_contact.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        emailing = mommy.make(models.Emailing, newsletter=newsletter)
        emailing.sent_to.add(contact)
        emailing.save()
        url = reverse('newsletters:view_online', args=[emailing.id, contact.uuid])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, contact.fullname)
        self.assertEqual(models.MagicLink.objects.count(), 2)
        magic_link0 = models.MagicLink.objects.all()[0]
        self.assertContains(response, reverse('newsletters:view_link', args=[magic_link0.uuid, contact.uuid]))
        self.assertEqual(magic_link0.url, '/this-link-without-prefix-in-template')
        magic_link1 = models.MagicLink.objects.all()[1]
        self.assertContains(response, reverse('newsletters:view_link', args=[magic_link1.uuid, contact.uuid]))
        self.assertEqual(magic_link1.url, 'http://toto.fr/à-bientôt')

    def test_view_long_links(self):
        contact = mommy.make(models.Contact, email='toto@toto.fr', first_name='Emmet', last_name='Brown')
        short_link = "http://toto.fr/{0}".format("abcde" * 100)[:499]
        long_link = "http://toto.fr/{0}".format("abcde" * 100)  # >500 chars
        newsletter_data = {
            'subject': 'This is the subject',
            'content': '<p>Visit <a href="{0}">long link</a> <a href="{1}">long link</a></p>'.format(
                short_link, long_link
            ),
            'template': 'test/newsletter_no_link.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        emailing = mommy.make(models.Emailing, newsletter=newsletter)
        emailing.sent_to.add(contact)
        emailing.save()
        url = reverse('newsletters:view_online', args=[emailing.id, contact.uuid])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        self.assertEqual(models.MagicLink.objects.count(), 1)
        magic_link0 = models.MagicLink.objects.all()[0]
        self.assertContains(response, reverse('newsletters:view_link', args=[magic_link0.uuid, contact.uuid]))
        self.assertEqual(magic_link0.url, short_link)
        self.assertContains(response, long_link)

    def test_view_duplicate_links(self):
        contact = mommy.make(models.Contact, email='toto@toto.fr', first_name='Emmet', last_name='Brown')
        short_link = "http://toto.fr/abcde/"
        newsletter_data = {
            'subject': 'This is the subject',
            'content': '<p>Visit <a href="{0}">link1</a> <a href="{1}">link2</a></p>'.format(
                short_link, short_link
            ),
            'template': 'test/newsletter_no_link.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        emailing = mommy.make(models.Emailing, newsletter=newsletter)
        emailing.sent_to.add(contact)
        emailing.save()
        url = reverse('newsletters:view_online', args=[emailing.id, contact.uuid])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        self.assertEqual(models.MagicLink.objects.count(), 1)
        magic_link0 = models.MagicLink.objects.all()[0]
        self.assertContains(response, reverse('newsletters:view_link', args=[magic_link0.uuid, contact.uuid]))
        self.assertEqual(magic_link0.url, short_link)

    def test_view_duplicate_emailing(self):
        contact = mommy.make(models.Contact, email='toto@toto.fr', first_name='Emmet', last_name='Brown')
        short_link = "http://toto.fr/abcde/"
        newsletter_data = {
            'subject': 'This is the subject',
            'content': '<p>Visit <a href="{0}">link1</a></p>'.format(
                short_link
            ),
            'template': 'test/newsletter_no_link.html'
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        emailing1 = mommy.make(models.Emailing, newsletter=newsletter)
        emailing1.sent_to.add(contact)
        emailing1.save()
        emailing2 = mommy.make(models.Emailing, newsletter=newsletter)
        emailing2.sent_to.add(contact)
        emailing2.save()
        for emailing in (emailing1, emailing2):
            url = reverse('newsletters:view_online', args=[emailing.id, contact.uuid])
            response = self.client.get(url)
            self.assertEqual(200, response.status_code)
            magic_links = models.MagicLink.objects.filter(emailing=emailing)
            self.assertEqual(magic_links.count(), 1)
            magic_link0 = magic_links[0]
            self.assertContains(response, reverse('newsletters:view_link', args=[magic_link0.uuid, contact.uuid]))
            self.assertEqual(magic_link0.url, short_link)





# -*- coding: utf-8 -*-
"""
misc. templatetags
"""
from django import template

register = template.Library()


class CustomNode(template.Node):

    def render(self, context):
        context.dicts[1]['custom_list'] = [1, 2, 3]
        return ""


@register.tag
def fill_custom_list(parser, token):
    args = token.split_contents()
    return CustomNode()



# -*- coding: utf-8 -*-
"""
Default CMS application : It define a simple Article  # Just for compatibility
"""

from ...bs_forms import Form, ModelForm


class BootstrapForm(Form):  # pylint: disable=R0901
    """Deprecated: Just for compatibility"""
    pass


class BootstrapModelForm(ModelForm):  # pylint: disable=R0901
    """Deprecated: Just for compatibility"""
    pass



# -*- coding: utf-8 -*-

from unittest import skipIf

from django.conf import settings
from django.contrib.sites.models import Site
from django.test.utils import override_settings
from django.utils.translation import get_language

from ..settings import is_localized, is_multilang, get_article_class
from ..utils import make_locale_path
from . import BaseTestCase


class ArticleSlugTestCase(BaseTestCase):
    """test slug"""
    
    def tearDown(self):
        """after each test"""
        super(ArticleSlugTestCase, self).tearDown()
        site1 = Site.objects.all()[0]
        settings.SITE_ID = site1.id
    
    def test_create_article_same_title(self):
        """test slug with duplicated titles"""
        article_class = get_article_class()
        article1 = article_class.objects.create(title="Titre de l'article")
        for number in range(12):
            article2 = article_class.objects.create(title=article1.title)
            self.assertNotEqual(article1.slug, article2.slug)
            self.assertEqual(article1.title, article2.title)
        response = self.client.get(article2.get_absolute_url())
        self.assertEqual(200, response.status_code)
        response = self.client.get(article1.get_absolute_url())
        self.assertEqual(200, response.status_code)
            
    def test_create_article_same_different_sites(self):
        """test slug on different sites"""
        article_class = get_article_class()
        article1 = article_class.objects.create(title="Titre de l'article")
        
        site1 = Site.objects.all()[0]
        site2 = Site.objects.create(domain='hhtp://test2', name="Test2")
        settings.SITE_ID = site2.id
        
        article2 = article_class.objects.create(title=article1.title)
        self.assertNotEqual(article1.slug, article2.slug)
        self.assertEqual(article1.title, article2.title)
        
        response = self.client.get(article1.get_absolute_url())
        self.assertEqual(404, response.status_code)
        
        response = self.client.get(article2.get_absolute_url())
        self.assertEqual(200, response.status_code)
        
        settings.SITE_ID = site1.id
        response = self.client.get(article1.get_absolute_url())
        self.assertEqual(200, response.status_code)
        
    @skipIf(not is_localized() or not is_multilang(), "not localized")
    def test_create_lang(self):
        """test slug with language"""
        
        article_class = get_article_class()
        article1 = article_class.objects.create(title="Titre de l'article")
        article2 = article_class.objects.create(title=article1.title)
        self.assertNotEqual(article1.slug, article2.slug)
        self.assertEqual(article1.title, article2.title)
        
        trans_lang = settings.LANGUAGES[1][0]
            
        setattr(article1, 'title_'+trans_lang, 'This is the title')
        article1.save()
        
        setattr(article2, 'title_'+trans_lang, getattr(article1, 'title_'+trans_lang))
        article2.save()
        
        article1 = article_class.objects.get(id=article1.id)
        article2 = article_class.objects.get(id=article2.id)
        
        self.assertEqual(getattr(article1, 'title_'+trans_lang), getattr(article2, 'title_'+trans_lang))
        self.assertNotEqual(getattr(article1, 'slug_'+trans_lang), getattr(article2, 'slug_'+trans_lang))
        
    def _get_localized_slug(self, slug):
        """get localized slug"""
        if is_localized():
            return make_locale_path(slug, get_language())
        return slug
    
    def test_create_article_html_in_title(self):
        """create article with html tag in title"""
        article_class = get_article_class()
        article1 = article_class.objects.create(title="<h1>Titre de l'article</h1>")
        response = self.client.get(article1.get_absolute_url())
        self.assertEqual(200, response.status_code)
        
        expected_title = self._get_localized_slug("/titre-de-larticle/")
        self.assertEqual(article1.get_absolute_url(), expected_title)
        
    def test_create_article_complex_html_in_title(self):
        """create article with html tag in title"""
        article_class = get_article_class()
        article1 = article_class.objects.create(title="<p><h2>Titre de <b>l'article</b><h2><div></div></p>")
        response = self.client.get(article1.get_absolute_url())
        self.assertEqual(200, response.status_code)
        expected_title = self._get_localized_slug("/titre-de-larticle/")
        self.assertEqual(article1.get_absolute_url(), expected_title)

    @override_settings(LANGUAGE_CODE='ru', LANGUAGES=(('ru', 'Russian'), ))
    def create_article_non_ascii_char(self):
        """create an article with russian characters"""
        title = "Миниальбом"
        article_class = get_article_class()
        article1 = article_class.objects.create(title=title)
        response = self.client.get(article1.get_absolute_url())
        self.assertEqual(200, response.status_code)



# -*- coding: utf-8 -*-
"""
coop_cms manage compatibilty with django and python versions
"""

import json
import sys

from django.conf import settings
from django.template import RequestContext, Context

from html.parser import HTMLParser as BaseHTMLParser


class HTMLParser(BaseHTMLParser):
    def __init__(self):
        BaseHTMLParser.__init__(self, convert_charrefs=False)


try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    MiddlewareMixin = object


def make_context(request, context_dict, force_dict=True):
    """"""
    if force_dict:
        context = dict(context_dict)
        if request:
            context['request'] = request
            context['MEDIA_URL'] = settings.MEDIA_URL
            context['user'] = request.user
    else:
        if request:
            context = RequestContext(request, context_dict)
        else:
            context = Context(context_dict)
    return context


def get_response_json(response):
    if sys.version_info[0] < 3:
        return json.loads(response.content)
    else:
        return response.json()


def is_authenticated(user):
    if callable(user.is_authenticated):
        return user.is_authenticated()
    return user.is_authenticated


def is_anonymous(user):
    return not is_authenticated(user)



# -*- coding: utf-8 -*-
"""models"""

from datetime import datetime
import uuid

from django.conf import settings

from django.contrib.sites.models import Site
from django.db import models
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from ...models import Newsletter


class Contact(models.Model):
    email = models.EmailField(verbose_name=_('email'), db_index=True)
    last_name = models.CharField(verbose_name=_('last name'), max_length=200, blank=True, default='')
    first_name = models.CharField(verbose_name=_('first name'), max_length=200, blank=True, default='')
    email_verified = models.BooleanField(_("email verified"), default=False)
    uuid = models.CharField(verbose_name=_('unique identifier'), max_length=200, blank=True, default='', db_index=True)
    favorite_language = models.CharField(_("favorite language"), max_length=10, default="", blank=True)

    def __str__(self):
        return self.email

    class Meta:
        verbose_name = _('Contact')
        verbose_name_plural = _('Contacts')

    @property
    def fullname(self):
        elements = [self.first_name, self.last_name]
        safe_elements = [elt for elt in elements if elt]
        return ' '.join(safe_elements)

    def save(self, *args, **kwargs):
        """save"""
        return_value = super().save(*args, **kwargs)
        if not self.uuid:
            name = '{0}-contact-{1}-{2}'.format(settings.SECRET_KEY, self.id, self.email)
            self.uuid = uuid.uuid5(uuid.NAMESPACE_URL, '{0}'.format(name))
            return super().save()
        return return_value


class SubscriptionType(models.Model):
    """Subscription type: a mailing list for example"""
    name = models.CharField(max_length=100, verbose_name=_("name"))
    site = models.ForeignKey(Site, blank=True, null=True, on_delete=models.CASCADE, related_name='+')
    order_index = models.IntegerField(default=0, verbose_name=_("order index"))
    allowed_on_sites = models.ManyToManyField(Site, blank=True, related_name='+')
    from_email = models.CharField(max_length=100, blank=True, default="", verbose_name=_("From email"))
    lang = models.CharField(_("language"), max_length=5, default="", blank=True)
    subject_prefix = models.CharField(
        max_length=100, blank=True, default="", verbose_name=_("préfixe du sujet"),
        help_text=_('Éviter de mettre tout en majuscule -> SPAM')
    )

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Subscription type')
        verbose_name_plural = _('Subscription types')
        ordering = ('order_index',)

    def get_domain_url_prefix(self):
        """domain url prefix"""
        if self.site:
            domain_protocol = getattr(settings, "COOP_CMS_DOMAIN_PROTOCOL", "http://")
            return domain_protocol + self.site.domain
        return ""


class Subscription(models.Model):
    """contact is subscribing to a mailing list"""
    subscription_type = models.ForeignKey(SubscriptionType, on_delete=models.CASCADE)
    contact = models.ForeignKey(Contact, on_delete=models.CASCADE)

    accept_subscription = models.BooleanField(
        _("accept subscription"), default=False,
        help_text=_('Keep this checked if you want to receive our newsletter.')
    )

    subscription_date = models.DateTimeField(blank=True, default=None, null=True)
    unsubscription_date = models.DateTimeField(blank=True, default=None, null=True)
    unsubscription_reason = models.TextField(blank=True, default="", verbose_name=_("unsubscription reason"))

    def __str__(self):
        return "{0} {1}".format(self.subscription_type, self.contact)


class Emailing(models.Model):
    """configuration on an emailing"""

    STATUS_EDITING = 1
    STATUS_SCHEDULED = 2
    STATUS_SENDING = 3
    STATUS_SENT = 4

    STATUS_CHOICES = (
        (STATUS_EDITING, _('Edition in progress')),
        (STATUS_SCHEDULED, _('Sending is scheduled')),
        (STATUS_SENDING, _('Sending in progress')),
        (STATUS_SENT, _('Sent')),
    )

    subscription_type = models.ForeignKey(SubscriptionType, on_delete=models.CASCADE)
    newsletter = models.ForeignKey(Newsletter, on_delete=models.CASCADE)
    send_to = models.ManyToManyField(Contact, blank=True, related_name="emailing_to_be_received")
    sent_to = models.ManyToManyField(Contact, blank=True, related_name="emailing_received")
    opened_emails = models.ManyToManyField(Contact, blank=True, related_name="emailing_opened")
    unsub = models.ManyToManyField(Contact, blank=True, related_name="emailing_unsub")
    status = models.IntegerField(default=STATUS_EDITING, choices=STATUS_CHOICES)

    creation_dt = models.DateTimeField(_(u"creation date"), blank=True, default=None, null=True)
    scheduling_dt = models.DateTimeField(_(u"scheduling date"), blank=True, default=None, null=True)
    sending_dt = models.DateTimeField(_(u"sending date"), blank=True, default=None, null=True)

    def __str__(self):
        return "{0} - {1} contacts".format(self.newsletter.subject, self.send_to.count())

    class Meta:
        verbose_name = _('emailing')
        verbose_name_plural = _('emailings')

    def save(self, *args, **kwargs):
        """save"""
        if self.creation_dt is None:
            self.creation_dt = datetime.now()

        if self.status == Emailing.STATUS_SENT and self.sending_dt is None:
            self.sending_dt = datetime.now()

        return super().save(*args, **kwargs)

    def get_domain_url_prefix(self):
        return self.subscription_type.get_domain_url_prefix()

    @property
    def from_email(self):
        return self.subscription_type.from_email

    @property
    def lang(self):
        return self.subscription_type.lang

    def get_absolute_url(self):
        if self.newsletter:
            return reverse('coop_cms_view_newsletter', args=[self.newsletter.id])
        return None


class MagicLink(models.Model):
    """A tracking link"""

    class Meta:
        verbose_name = _('Magic link')
        verbose_name_plural = _('Magic links')

    emailing = models.ForeignKey(Emailing, on_delete=models.CASCADE)
    url = models.URLField(max_length=500)
    visitors = models.ManyToManyField(Contact, blank=True)
    uuid = models.CharField(max_length=100, blank=True, default='', db_index=True)

    def __unicode__(self):
        return self.url

    def save(self, *args, **kwargs):
        """save"""
        return_value = super().save(*args, **kwargs)
        if not self.uuid:
            name = '{0}-magic-link-{1}-{2}'.format(settings.SECRET_KEY, self.id, self.url)
            self.uuid = uuid.uuid5(uuid.NAMESPACE_URL, name)
            return super().save()
        return return_value



# -*- coding: utf-8 -*-
"""emailing views"""

import datetime
import os.path

from django.urls import reverse
from django.http import HttpResponse, HttpResponseRedirect, Http404
from django.shortcuts import render, get_object_or_404
from django.template.loader import get_template

from ...utils import redirect_to_language
from wsgiref.util import FileWrapper

from . import models
from . import forms
from .utils import get_emailing_context, patch_emailing_html


def view_link(request, link_uuid, contact_uuid):
    """view magic link"""
    link = get_object_or_404(models.MagicLink, uuid=link_uuid)
    try:
        user = models.Contact.objects.get(uuid=contact_uuid)
        link.visitors.add(user)
    except models.Contact.DoesNotExist:
        pass
    return HttpResponseRedirect(link.url)


def unregister_contact(request, emailing_id, contact_uuid):
    """contact unregister from emailing list"""
    contact = get_object_or_404(models.Contact, uuid=contact_uuid)
    try:
        emailing = models.Emailing.objects.get(id=emailing_id)
    except models.Emailing.DoesNotExist:
        raise Http404

    if request.method == "POST":
        if 'unregister' in request.POST:
            form = forms.UnregisterForm(request.POST)
            if form.is_valid():
                if emailing and emailing.subscription_type:
                    active_subscriptions = models.Subscription.objects.filter(
                        contact=contact,
                        subscription_type=emailing.subscription_type,
                        accept_subscription=True
                    )
                    if active_subscriptions.count() == 0:
                        subscription = models.Subscription.objects.create(
                            contact=contact,
                            subscription_type=emailing.subscription_type,
                            subscription_date=datetime.datetime.now()
                        )
                        active_subscriptions = [subscription]
                    for subscription in active_subscriptions:
                        subscription.accept_subscription = False
                        subscription.unsubscription_date = datetime.datetime.now()
                        subscription.unsubscription_reason = form.cleaned_data.get('reason', '')
                        subscription.save()
                    emailing.unsub.add(contact)
                    emailing.save()
                return render(
                    request,
                    'newsletters/public/unregister_done.html',
                    {
                        'contact': contact,
                        'emailing': emailing,
                        'form': form,
                        'unregister': True,
                    }
                )
            else:
                pass  # not valid : display with errors

        else:
            return render(
                request,
                'newsletters/public/unregister_done.html',
                {
                    'contact': contact,
                    'emailing': emailing,
                }
            )
    else:
        form = forms.UnregisterForm()

    return render(
        request,
        'newsletters/public/unregister_confirm.html',
        {
            'contact': contact,
            'emailing': emailing,
            'form': form,
        }
    )


def view_emailing_online(request, emailing_id, contact_uuid):
    """view an emailing online"""
    contact = get_object_or_404(models.Contact, uuid=contact_uuid)
    emailing = get_object_or_404(models.Emailing, id=emailing_id)
    context = get_emailing_context(emailing, contact)
    the_template = get_template(emailing.newsletter.get_template_name())
    html_text = the_template.render(context)
    html_text = patch_emailing_html(html_text, emailing, contact)
    return HttpResponse(html_text)


def view_emailing_online_lang(request, emailing_id, contact_uuid, lang):
    """view an emailing in a given lang"""
    url = reverse("newsletters:view_online", args=[emailing_id, contact_uuid])
    return redirect_to_language(url, lang)


def email_tracking(request, emailing_id, contact_uuid):
    """handle download of email opening tracking image"""
    emailing = get_object_or_404(models.Emailing, id=emailing_id)
    contact = get_object_or_404(models.Contact, uuid=contact_uuid)

    emailing.opened_emails.add(contact)
    emailing.save()

    dir_name = os.path.dirname(os.path.abspath(__file__))
    file_name = os.path.join(dir_name, "files/email-tracking.png")
    response = HttpResponse(FileWrapper(open(file_name, 'rb')), content_type='image/png')
    response['Content-Length'] = os.path.getsize(file_name)
    return response



# -*- coding: utf-8 -*-
"""
How to configure the demo CMS
"""

COOP_CMS_ARTICLE_CLASS = 'coop_cms.apps.demo_cms.models.Article'
COOP_CMS_ARTICLE_FORM = 'coop_cms.apps.demo_cms.forms.ArticleForm'

COOPBAR_MODULES = ('coop_cms.apps.demo_cms.my_coop_bar',)
DJALOHA_LINK_MODELS = ('demo_cms.Article',)
COOP_CMS_ARTICLE_LOGO_SIZE = "128x128"

COOP_CMS_ARTICLE_TEMPLATES = 'coop_cms.apps.demo_cms.get_article_templates'
# COOP_CMS_ARTICLE_TEMPLATES = (
#     ('standard.html', 'Standard'),
#     ('homepage.html', 'Homepage'),
#     ('blog.html', 'Blog'),
# )

COOP_CMS_SITE_PREFIX = 'http://127.0.0.1:8000'
COOP_CMS_FROM_EMAIL = '"Your name" <your@email.com>'
COOP_CMS_TEST_EMAILS = ('"Your name" <your@email.com>',)
COOP_CMS_NEWSLETTER_TEMPLATES = (
    ('basic_newsletter.html', 'Basic'),
    ('special_newsletter.html', 'With categories'),
    ('sortable_newsletter.html', 'Sortable categories'),
)
COOP_CMS_NEWSLETTER_FORM = 'coop_cms.apps.demo_cms.forms.SortableNewsletterForm'



# -*- coding: utf-8 -*-
"""for unit-testing"""

from django.urls import path

from . import views


urlpatterns = [
    path('coop-cms-testclass/', views.TestClassListView.as_view(), name='coop_cms_testapp_list'),
    path('works/<int:pk>/cms_edit/', views.TestClassEditView.as_view(), name='coop_cms_testapp_edit'),
    path('works/<int:pk>/', views.TestClassDetailView.as_view(), name='coop_cms_testapp_detail'),
    path('works/cms_edit/', views.TestClassFormsetEditView.as_view(), name='coop_cms_testapp_formset_edit'),
    path('works/', views.TestClassFormsetView.as_view(), name='coop_cms_testapp_formset'),
]



# -*- coding: utf-8 -*-

from django.urls import path
from django.views.generic.base import TemplateView

from .views import EmailRegistrationView, EmailActivationView

urlpatterns = [
    path(
        'activate/complete/',
        TemplateView.as_view(template_name='django_registration/activation_complete.html'),
        name='django_registration_activation_complete'
    ),
    # Activation keys get matched by \w+ instead of the more specific
    # [a-fA-F0-9]{40} because a bad activation key should still get to the view;
    # that way it can return a sensible "invalid key" message instead of a
    # confusing 404.
    path(
        'activate/<str:activation_key>/',
        EmailActivationView.as_view(),
        name='registration_activate'
    ),
    path(
        'register/',
        EmailRegistrationView.as_view(),
        name='registration_register'
    ),
    path(
        'register/complete/',
        TemplateView.as_view(template_name='django_registration/registration_complete.html'),
        name='django_registration_complete'
    ),
    path(
        'register/closed/',
        TemplateView.as_view(template_name='django_registration/registration_closed.html'),
        name='django_registration_disallowed'
    ),
]



# -*- coding: utf-8 -*-

from django.db import models, migrations
import datetime
import coop_cms.models
import django.utils.timezone
from django.conf import settings
import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('coop_cms', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('sites', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Article',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', django_extensions.db.fields.CreationDateTimeField(default=django.utils.timezone.now, verbose_name='created', editable=False, blank=True)),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(default=django.utils.timezone.now, verbose_name='modified', editable=False, blank=True)),
                ('slug', models.CharField(unique=True, max_length=100, db_index=True)),
                ('title', models.TextField(default=b'', verbose_name='title', blank=True)),
                ('subtitle', models.TextField(default=b'', verbose_name='subtitle', blank=True)),
                ('content', models.TextField(default=b'', verbose_name='content', blank=True)),
                ('publication', models.IntegerField(default=1, verbose_name='publication', choices=[(0, 'Draft'), (1, 'Published'), (2, 'Archived')])),
                ('template', models.CharField(default=b'', max_length=200, verbose_name='template', blank=True)),
                ('logo', models.ImageField(default=b'', null=True, upload_to=coop_cms.models.get_logo_folder, blank=True)),
                ('temp_logo', models.ImageField(default=b'', null=True, upload_to=coop_cms.models.get_logo_folder, blank=True)),
                ('summary', models.TextField(default=b'', verbose_name='Summary', blank=True)),
                ('in_newsletter', models.BooleanField(default=True, help_text='Make this article available for newsletters.', verbose_name='In newsletter')),
                ('headline', models.BooleanField(default=False, help_text='Make this article appear on the home page', verbose_name='Headline')),
                ('publication_date', models.DateTimeField(default=datetime.datetime.now, verbose_name='Publication date')),
                ('author', models.ForeignKey(default=None, blank=True, to=settings.AUTH_USER_MODEL, null=True, on_delete=models.CASCADE)),
                ('category', models.ForeignKey(related_name='demo_cms_article_rel', default=None, blank=True, to='coop_cms.ArticleCategory', null=True, verbose_name='Category', on_delete=models.CASCADE)),
                ('homepage_for_site', models.ForeignKey(related_name='homepage_article', default=None, blank=True, to='sites.Site', null=True, verbose_name='Homepage for site', on_delete=models.CASCADE)),
                ('sites', models.ManyToManyField(default=[1], to='sites.Site', verbose_name='site')),
            ],
            options={
                'abstract': False,
                'verbose_name': 'article',
                'verbose_name_plural': 'articles',
            },
        ),
        migrations.CreateModel(
            name='ModeratedArticle',
            fields=[
            ],
            options={
                'proxy': True,
            },
            bases=('demo_cms.article',),
        ),
        migrations.CreateModel(
            name='PrivateArticle',
            fields=[
            ],
            options={
                'proxy': True,
            },
            bases=('demo_cms.article',),
        ),
    ]



# -*- coding: utf-8 -*-
"""test email sending"""

from django.urls import reverse

from model_mommy import mommy

from ....tests import BaseTestCase

from .. import models


class MagicLinkTest(BaseTestCase):

    def test_view_magic_link(self):
        contact = mommy.make(models.Contact,email='toto@toto.fr', last_name='Toto')
        emailing = mommy.make(models.Emailing)
        emailing.sent_to.add(contact)
        emailing.save()
        link = "http://www.google.fr"
        magic_link = models.MagicLink.objects.create(emailing=emailing, url=link)
        self.assertEqual(magic_link.visitors.count(), 0)
        response = self.client.get(reverse('newsletters:view_link', args=[magic_link.uuid, contact.uuid]))
        self.assertEqual(302, response.status_code)
        self.assertEqual(response['Location'], link)
        self.assertEqual(magic_link.visitors.count(), 1)
        self.assertEqual(magic_link.visitors.all()[0], contact)
